/* #######################
 *  DISPLAY DRIVER SKETCH 
 * #######################
 * 
 * Project Title: Puzzle-Bot
 * Purpose: Use a Dobot Magician robot arm to solve a 4x4 sliding puzzle
 * Microcontroller: Arduino Mega 2560
 * 
 * File: DisplayDriver7
 * Date: 8/9/23
 * 
 * Written by: Dave Goodsell & Sparkfun
 * dsgoodsell@verizon.net
 * 
 */
//****************************************************************************************************
//ALERT, ALERT! THIS SKETCH WILL NOT DOWNLOAD UNLESS THE "DISPLAY RESET" SWITCH IS SET AT "MAN(PRGM)"
//****************************************************************************************************
//COM10  COM10   COM10   COM10
                
//Ver.4: Stable                
//Ver.5: Code cleanup. 
//Ver.5a: Use this instead of Ver.6 because 6 has too much garbage now.
//Ver.7: Continuing, need to download again, DONE


 //******************************************************************
 //    Required paint code from Sparkfun - modify carefully
 //******************************************************************
  /* Serial Paint (RGB Panel Paint)
   by: Jim Lindblom @ SparkFun Electronics
   date: December 9, 2013
   modified by: Ho Yun "Bobby" Chan @ SparkFun Electronics
   date: May 10, 2018

   license: Beerware. Please use, reuse, and modify this sketch.
   If you find it useful, and we meet some day, we can share a beer.

   Description: This sketch serves as a fun, interactive demo
   for the following RGB Panels.

     32x64, 1:16 scan | https://www.sparkfun.com/products/14718
     32x32, 1:16 scan | https://www.sparkfun.com/products/12584
     16x32, 1:8 scan | https://www.sparkfun.com/products/retired/12583

   After loading the sketch, and hooking everything up, you can
   control the panel from your serial monitor (we actually
   recommend using a real serial program, like Tera Term). 9600
   bps.

   Your cursor should blink the active color. Controls are:
     Movement: w, a, s, d (up, down, left, right)
     Draw Pixel: Spacebar
     Erase Pixel: e
     Erase Screen: E
     Fill screen active color: f
     Red value up: R (values between 0 and 7)
     Red value down: r
     Green value up/down: G/g
     Blue value up/ down: B/b
     Copy color: z copies a color under the cursor
     Draw line: press v to place starting point. Then move cursor
     to endpoint and press v again.
     Draw Box: press x or X to place first corner. Then move
     your cursor to where you want the diagonal corner. Then press
     either x for a no-fill box, or X for a filled box.
     Draw Circle: press c or C to place the _center_ of the circle.
     Then move your cursor to where you want the outside edge of
     your circle to be. Then press c for a no-fill circle or C for
     a filled circle.
     Text mode: press t to go to text mode. Now any characters
     received will be displayed on the panel. It'll wrap around
     from one line to the next, but not from bottom to top.
     Press ` (above Tab / left of 1) to exit text mode.
     Print: press p to print an array

   Check out our hookup guide for help wiring it all up:
     https://learn.sparkfun.com/tutorials/rgb-panel-hookup-guide

   You'll need to download and install the RGBmatrixPanel and
   Adafruit_GFX libraries:
     https://github.com/adafruit/RGB-matrix-Panel
     https://github.com/adafruit/Adafruit-GFX-Library

   This sketch uses the RGBmatrixPanel and Adafruit_GFX libraries,
   written by Limor Fried/Ladyada & Phil Burgess/PaintYourDragon
   for Adafruit Industries. Released under the BSD license.
*/

//////////////////////
// Library Includes //
//////////////////////
#include <Adafruit_GFX.h>   // Core graphics library
#include <RGBmatrixPanel.h> // Hardware-specific library
#include "bitmap.h"

/////////////////////
// Hardware Hookup //
/////////////////////
// R0, G0, B0, R1, G1, B1 should be connected to pins
// 2, 3, 4, 5, 6, and 7 respectively. Their pins aren't defined,
// because they're controlled directly in the library. These pins
// can be moved (somewhat):

// CLK be on PORTB!
// CLK can be moved but must remain on PORTB(8, 9, 10, 11, 12, 13)
#define CLK 11
#define LAT 10
#define OE  9

#define A   A0
#define B   A1
#define C   A2
#define D   A3 // Comment this line out if you're using a 32x16

////////////////////////////////////////////////
/////   Instantiate RGBmatrixPanel Class   /////
////////////////////////////////////////////////

/* - One of these should be commented out!
   - Also, make sure to adjust the saved image in the <bitmap.h> file.*/

/* ========== For 32x64 LED panels: ==========
  You MUST use an Arduino Mega2560 with 32x64 size RGB Panel */
//RGBmatrixPanel matrix(A, B, C, D, CLK, LAT, OE, false, 64); // 32x64

/* ========== For 32x32 LED panels: ========== */
RGBmatrixPanel matrix(A, B, C, D, CLK, LAT, OE, false); // 32x32

/* ==========  For 32x16 LED panels: ========== */
//RGBmatrixPanel matrix(A, B, C, CLK, LAT, OE, false); // 32x16

///////////////////////////
// Paint Program Globals //
///////////////////////////
// Enum UP, DOWN, LEFT, RIGHT to make programming easier:
enum direction {
  UP,    // 0
  DOWN,  // 1
  LEFT,  // 2
  RIGHT  // 3
};

int8_t cursorX = 0;  // Cursor x position, initialize left
int8_t cursorY = 0;  // Cursor y position, initialize top
int16_t oldColor = 0; // Keep track of color under cursor
int8_t red = 1;   // Red paint value
int8_t blue = 1;  // Blue paint value
int8_t green = 1; // Green paint value

// Enum shape drawing mode:
enum shapeMode {
  NONE,   // 0
  LINE,   // 1
  CIRCLE, // 2
  RECT    // 3
};
int8_t activeDraw = NONE; // Keeps track of whether we're drawing a shape
int16_t oldCenterColor; // For circle, keeps track of color under cursor
int8_t shapeX0;  // x0 of line or box, center of circle
int8_t shapeY0;  // y0 of line or box, center of circle

#define EXIT_CHAR '`'
// Timing stuff:
#define BLINK_RATE 250 // blink rate in ms


//****************************************************************************
//   DO NOT ERASE ANY OF THE ABOVE. SOME OF IT IS USED.
//****************************************************************************
   
byte num1Array[8][2] = {
   3,2,\
   4,1,\
   4,2,\
   4,3,\
   4,4,\
   4,5,\
   4,6,\
   33,33
};

byte num2Array[16][2] = {
   2,2,\
   3,1,\
   4,1,\
   5,1,\
   6,2,\
   6,3,\
   5,4,\
   4,4,\
   3,4,\
   2,5,\
   2,6,\
   3,6,\
   4,6,\
   5,6,\
   6,6,\
   33,33
};

byte num3Array[15][2] = {

  2,2,\
  3,1,\
  4,1,\
  5,1,\
  6,2,\
  6,3,\
  6,4,\
  6,5,\
  5,6,\
  4,6,\
  3,6,\
  2,5,\
  4,3,\
  5,3,\
  33,33
};

byte num4Array[13][2] = {

  2,1,\
  2,2,\
  2,3,\
  3,3,\
  4,3,\
  5,3,\
  6,3,\
  5,1,\
  5,2,\
  5,4,\
  5,5,\
  5,6,\
  33,33
};

byte num5Array[17][2] = {

2,1,\
3,1,\
4,1,\
5,1,\
6,1,\
2,2,\
2,3,\
3,3,\
4,3,\
5,3,\
6,4,\
6,5,\
2,5,\
3,6,\
4,6,\
5,6,\
33,33  
};

byte num6Array[17][2] = {

  2,2,\
  2,3,\
  2,4,\
  2,5,\
  3,1,\
  4,1,\
  5,1,\
  5,1,\
  3,3,\
  4,3,\
  5,3,\
  6,4,\
  6,5,\
  3,6,\
  4,6,\
  5,6,\
  33,33
};

byte num7Array[11][2] = {

  2,1,\
  3,1,\
  4,1,\
  5,1,\
  6,1,\
  6,2,\
  5,3,\
  4,4,\
  3,5,\
  2,6,\
  33,33
};

byte num8Array[18][2] = {

  3,1,\
  4,1,\
  5,1,\
  6,2,\
  6,3,\
  6,4,\
  6,5,\
  3,6,\
  4,6,\
  5,6,\
  2,2,\
  2,3,\
  2,4,\
  2,5,\
  3,3,\
  4,3,\
  5,3,\
  33,33
};

byte num9Array[17][2] = {

  3,1,\
  4,1,\
  5,1,\
  6,2,\
  6,3,\
  6,4,\
  6,5,\
  3,6,\
  4,6,\
  5,6,\
  2,5,\
  2,2,\
  2,3,\
  3,3,\
  4,3,\
  5,3,\
  33,33
};

byte num10Array[17][2] = {
2,1,\
2,2,\
2,3,\
2,4,\
2,5,\
2,6,\
4,2,\
5,1,\
6,2,\
6,3,\
6,5,\
6,4,\
5,6,\
4,5,\
4,3,\
4,4,\
33,33
};

byte num11Array[15][2] = {
   2,2,\
   3,1,\
   3,2,\
   3,3,\
   3,4,\
   3,5,\
   3,6,\
   5,2,\
   6,1,\
   6,2,\
   6,3,\
   6,4,\
   6,5,\
   6,6,\
   33,33
};

byte num12Array[19][2] = {

2,1,\
2,2,\
2,3,\
2,4,\
2,5,\
2,6,\

2,6,\
4,2,\
5,1,\
6,2,\
6,3,\
5,4,\
4,5,\
4,5,\
4,6,\
4,6,\
5,6,\
6,6,\
33,33
};

byte num13Array[17][2] = {
   4,3,\
   4,1,\
   5,1,\
   6,2,\
   6,3,\
   6,4,\
   6,5,\
   5,6,\
   4,6,\
   5,3,\
   2,1,\
   2,2,\
   2,3,\
   2,4,\
   2,5,\
   2,6,\
   33,33
};

byte num14Array[18][2] = {
  
2,1,\
2,2,\
2,3,\
2,4,\
2,5,\
2,6,\
4,1,\
4,2,\
4,3,\
4,3,\
5,3,\
6,1,\
6,2,\
6,3,\
6,4,\
6,5,\
6,6,\
33,33
};

byte num15Array[20][2] = {
2,1,\
2,2,\
2,3,\
2,4,\
2,5,\
2,6,\

4,1,\
4,1,\
5,1,\
6,1,\  
4,2,\
4,3,\
4,3,\
5,3,\
6,4,\
6,5,\
6,5,\
4,6,\
5,6,\
33,33
};

//END OF ARRAYS
//****************************************************

//Global Variables
byte LEDMatrixPosition[17];
int From;
int To;
int X;
int Y;
int x;
int y;
int Num;
int Xpos;
int Ypos;
int Xmove = 0;  //not used, eliminate later throughout program
int Ymove = 0;  //not used, eliminate later
int yesno;

int brightOnePin = 2;
int brightTwoPin = 3;
int brightFourPin = 4;
int brightEightPin = 5;

int button = 53;
int button1 = 51;
byte receivingBuffer[54];
int idx;

int RequestToSendSolveMoves = 41;     
int RequestToSendStartingArray = 43;
int DisplayReadyToReceiveData = 45;
int RequestToSendMovesToDobot = 47;
int blankEaselPin = 23;
int currentEaselPinState = 0;
int lastEaselPinState = 0;
bool easelPinState;
bool noEaselBlankingFlag = 0;

bool brightnessFlag1;
bool brightnessFlag2;
bool brightnessFlag3;
bool brightnessFlag4;
bool brightnessFlag5;
bool brightnessFlag6;
bool brightnessFlag7;
bool brightnessFlag8;
bool brightnessFlag9;
bool brightnessFlag10;
bool brightnessFlag11;
bool brightnessFlag12;
bool brightnessFlag13;
bool brightnessFlag14;
bool brightnessFlag15;

int Digit;
//int numToBrighten;
int posXToBrighten;
int posYToBrighten; 

//******************************************************
//STARTING MATRIX
byte startingMatrixArray[17] = {
 0,5,3,16,6,8,7,4,15,14,12,10,13,1,9,11,2
};

//******************************************************
//matrix coordinates of the upper-left corners of each square in the LED display panel, squares 1 to 16
byte coordinatesInLEDMatrix[17][2] = {
  0,0,\
  1,1,\
  8,1,\
  15,1,\
  22,1,\
  1,8,\
  8,8,\
  15,8,\
  22,8,\
  1,15,\
  8,15,\
  15,15,\
  22,15,\
  1,22,\
  8,22,\
  15,22,\
  22,22
};


//*********************************************************************************************************
//*********************************************************************************************************
void setup()
{
pinMode(button, INPUT_PULLUP);    //send moves
pinMode(button1, INPUT_PULLUP);   //send array

pinMode(RequestToSendSolveMoves, INPUT);       //pin41    The moves start after XMTR receives DTR from RCVR
pinMode(RequestToSendStartingArray, INPUT);    //pin43    The array gets sent after a DTR is received
pinMode(DisplayReadyToReceiveData, OUTPUT);    //pin45
pinMode(RequestToSendMovesToDobot, INPUT);     //pin47
pinMode(blankEaselPin, INPUT);                 //pin23

pinMode(brightOnePin, INPUT);   //making a number brighter on the display as it is solved
pinMode(brightTwoPin, INPUT);
pinMode(brightFourPin, INPUT);
pinMode(brightEightPin, INPUT);

matrix.begin();       // Initialize the matrix.
blankEasel();         // Blank screen

Serial.begin(115200);   // Start serial
Serial2.begin(9600);  //data from "solve" Arduino
}

//*********************************************************************************************************
//*********************************************************************************************************
void loop(){

//handshake loop
//****************************************************
  
//get ready for Moves to arrive 
//******************************
if ((digitalRead(RequestToSendSolveMoves)) == HIGH)
{
digitalWrite(DisplayReadyToReceiveData, HIGH);
receiveFromAndToMoves();
digitalWrite(DisplayReadyToReceiveData, LOW); 
}

//******************************************************

//get ready for Array to arrive
//*****************************
if ((digitalRead(RequestToSendStartingArray)) == HIGH)
{
digitalWrite(DisplayReadyToReceiveData, HIGH); 
delay(500);  //allow time for data to come in
receiveStartingMatrixArray();
digitalWrite(DisplayReadyToReceiveData, LOW);
}

/*
//watch for blankEaselPin HIGH pulse, then blankEasel() just once
//****************************************************************
//This routine cuts off the display at the end of matching... BAD!
//I do
if ((digitalRead(blankEaselPin)) == HIGH)
 {
  if (easelPinState == 1)
    {
    return;
    }
  else
    {
    blankEasel();
    easelPinState = 1;
    delay(50);
    }
 }
else
 {
  easelPinState = 0;
 }
*/

}  //end of main loop()


//**********************************************************************************************************************
//**********************************************************************************************************************
//                                      LED DISPLAY FUNCTIONS
//**********************************************************************************************************************
//**********************************************************************************************************************

void receiveFromAndToMoves(){
  
//blankEasel(); // Blank screen
//drawFrame();
//blinkCursor(false); // Turn cursor off 
//drawStartingMatrix();   //and draw the StartingMatrix on the display
//loadLEDMatrixWithStartingMatrixArray();

do
{
//from & to move commands
// MV,13,09,  = move(MV) position 13 to 9

x=0;
idx = 0;

//******************************************************************************
//    START OF THE BIG DO LOOP TO RECEIVE MOVES FROM "PUZZLE SOLVER" ARDUINO
//******************************************************************************
do 
{
if (Serial2.available())
{
 byte c = Serial2.read();
 Serial.print(c);
 receivingBuffer[idx] = c;
 idx++;
 if (idx>=9)
 {
  x=1;
 }
}
}
while(x != 1);

Serial.println("");

for (idx=0;idx<=8;idx++)
  {
  Serial.print(receivingBuffer[idx]);
  Serial.print(" ");
  }

Serial.println("");

convertToFromTo();

Serial.print("From: ");
Serial.println(From);
Serial.print("To: ");
Serial.println(To);

if (From == 17)
  {
  blinkOuterFrame(); 
  return;
  }

if (receivingBuffer[0] != 77)
  {
  posXToBrighten = From;
  posYToBrighten = To;
  brightenDigit(Digit);
  }
else
  {
  movePos(From,To); 
  }  

/*
if (From == 17)
  {
  blinkOuterFrame(); 
  return; 
  }
else
  {
  movePos(From,To);
  }
*/
  
}
while(1);   //END OF BIG DO LOOP
//**************************************************************

//do
//{}
//while(1);

}  //END RECEIVE AND PRINT FROM/TO MOVE

//**************************************************************
void receiveStartingMatrixArray(){

x = 0;
idx=0;

do
{
if (Serial2.available())
{
 byte c = Serial2.read();
 Serial.print(c);
 receivingBuffer[idx] = c;

 if (c == 90)   //ZZ, which is the final completed matrix to display
  {
  noEaselBlankingFlag = 1; 
  }
 else
  {
  noEaselBlankingFlag = 0;
  }
 
 idx++;
 if (idx>=54)
 {
  x=1;
 }
}
}
while(x != 1);

Serial.println(" ");

convertToStartingMatrixArray();  

Serial.print("final: ");
for (idx=0;idx<=16;idx++)
  {
  Serial.print(startingMatrixArray[idx]);
  Serial.print(" ");
  } 

blankEasel(); // Blank screen
drawFrame();
blinkCursor(false); // Turn cursor off 
drawStartingMatrix();   //and draw the StartingMatrix on the display
delay(2000);
if (blankEaselPin == LOW)
 {
 blankEasel();
 }
loadLEDMatrixWithStartingMatrixArray();

}

//*********************************************************************
void brightenDigit(int Dig){
//Dig,From,To

switch(Dig)
{
  case 1: drawNum1(From, To);break;
  case 2: drawNum2(From, To);break;
  case 3: drawNum3(From, To);break;
  case 4: drawNum4(From, To);break;
  case 5: drawNum5(From, To);break;
  case 6: drawNum6(From, To);break;
  case 7: drawNum7(From, To);break;
  case 8: drawNum8(From, To);break;
  case 9: drawNum9(From, To);break;
  case 10: drawNum10(From, To);break;
  case 11: drawNum11(From, To);break;
  case 12: drawNum12(From, To);break;
  case 13: drawNum13(From, To);break;
  case 14: drawNum14(From, To);break;
  case 15: drawNum15(From, To);break;
  default: break;  
}
  
}

//**********************************************************************
void blinkOuterFrame(){
  
resetBrightnessFlags();
  
for (int idx=1;idx<=4;idx++)
  {
  drawOuterFrame(0,5,0);
  delay(200);
  drawOuterFrame(0,0,0);
  delay(200);
  } 
}

//*********************************************************************
void convertToFromTo(){

//typ: MV,13,09,    or 02,07,11,(if we are brightning a digit)
if (receivingBuffer[0] != 77)
  {
  Digit = (((receivingBuffer[0]-48))*10) + ((receivingBuffer[1]) - 48);
  }
From = (((receivingBuffer[3]-48))*10) + ((receivingBuffer[4]) - 48);
To = (((receivingBuffer[6]-48))*10) + ((receivingBuffer[7]) - 48);
}

//***********************************************
void convertToStartingMatrixArray(){
int tens;
int units;
int number;
byte interimStartingMatrixArray[60];
//int idy;

//Serial.print
for (idx=3;idx<=51;idx+=3)
{
tens = (((receivingBuffer[idx+0]-48))*10);
units = ((receivingBuffer[idx+1]) - 48);
// idx+2 = the comma
number = (tens + units);
//Serial.print(number);
//Serial.print(",");
interimStartingMatrixArray[(idx/3)] = number;
}

//Serial.println(" ");
for (idx=1;idx<=17;idx++)
  {
  startingMatrixArray[idx-1] = interimStartingMatrixArray[idx];  
  //Serial.print(interimStartingMatrixArray[idx]);
  //Serial.print(" ");
  } 

}

//*********************************************************************
void drawStartingMatrix(){
for (int posInMatrix=1;posInMatrix<=16;posInMatrix++){
  Num = startingMatrixArray[posInMatrix];
  Xpos = coordinatesInLEDMatrix[posInMatrix][0]; 
  Ypos = coordinatesInLEDMatrix[posInMatrix][1];
  drawNumberOnBoard(Num,Xpos,Ypos);
  drawFrame();  //redraw Frame each time because the numbers obscure the upper frame line??? This should be fixed later on
}  
}

//*********************************************************************
void blinkCursor(boolean stat)
{
  if (stat) // Blink on:
    matrix.drawPixel(cursorX, cursorY, matrix.Color333(0,0,0));
  else      // Blink off
    matrix.drawPixel(cursorX, cursorY, 0);
}

//**********************************************************************
void movePos(int from, int to){
 
//The following is an important step to update the Master grade      
LEDMatrixPosition[to] = LEDMatrixPosition[from];  
LEDMatrixPosition[from] = 16;                    

//drawNumberOnBoard(Num,Xpos,Ypos);  
//I want Xpos & Ypos equal to the "to" position. coordinatesInLEDMatrix[17][2] = {X & Y}
drawNumberOnBoard(LEDMatrixPosition[to],coordinatesInLEDMatrix[to][0],coordinatesInLEDMatrix[to][1]);  
drawNumberOnBoard(16,coordinatesInLEDMatrix[from][0],coordinatesInLEDMatrix[from][1]); 
//drawFrame();
//blinkCursor(false); // Turn cursor off 
  
}

//*************************************************************
void loadLEDMatrixWithStartingMatrixArray(){
for (int idz=0;idz<=17;idz++)
{
LEDMatrixPosition[idz] = startingMatrixArray[idz];
}
}

//*****************************************************
void drawNumberOnBoard(int Num,int Xpos,int Ypos){

//delay(1000);
//for (int xx=1;xx<=15;xx++){
 
  switch(Num){
    case 1: {drawNum1(Xpos,Ypos);break;}
    case 2: {drawNum2(Xpos,Ypos);break;}
    case 3: {drawNum3(Xpos,Ypos);break;}
    case 4: {drawNum4(Xpos,Ypos);break;}
    case 5: {drawNum5(Xpos,Ypos);break;}
    case 6: {drawNum6(Xpos,Ypos);break;}
    case 7: {drawNum7(Xpos,Ypos);break;}
    case 8: {drawNum8(Xpos,Ypos);break;}
    case 9: {drawNum9(Xpos,Ypos);break;}
    case 10: {drawNum10(Xpos,Ypos);break;}
    case 11: {drawNum11(Xpos,Ypos);break;}
    case 12: {drawNum12(Xpos,Ypos);break;}
    case 13: {drawNum13(Xpos,Ypos);break;}
    case 14: {drawNum14(Xpos,Ypos);break;}
    case 15: {drawNum15(Xpos,Ypos);break;}
    case 16: {drawNum16(Xpos,Ypos);break;}
    delay(300);
}
//delay(2000);
}

//*********************************************************
//blank square 
void drawBlankSquare(int X,int Y, int Xmove, int Ymove){
 for (x=1+X+Xmove;x<=6+X+Xmove;x++)
 {
  for (y=1+Y+Ymove;y<=6+Y+Ymove;y++) 
  {
      drawDot(1);
      oldColor = matrix.Color333(0,0,0);
      matrix.drawPixel(x,y,oldColor);
      }
    
  }
 }

//*************************************************
void drawFrame(){

//vert lines
//*****************************  
for (int idx=0;idx<=4;idx++)
{
 for (int idy=1;idy<=29;idy++)
  {
  cursorX = 1 + (idx*7);
  cursorY = idy;
  drawDot(1);
  oldColor = matrix.Color333(0,1,0);
  matrix.drawPixel(cursorX, cursorY,oldColor);
  }
}

drawDot(1);
oldColor = matrix.Color333(0,0,0);
matrix.drawPixel(0,29,oldColor);
drawDot(1);
oldColor = matrix.Color333(0,0,0);
matrix.drawPixel(29,29,oldColor);

//horiz lines
//****************************
for (int idy=0;idy<=4;idy++)
{
 for (int idx=1;idx<=29;idx++)
  {
  cursorY = 1 + (idy*7);
  cursorX = idx;
  drawDot(1);
  oldColor = matrix.Color333(0,1,0);
  matrix.drawPixel(cursorX, cursorY,oldColor);
  }
}

//trying to eliminate cursors, kinda working??????
drawDot(1);
oldColor = matrix.Color333(0,0,0);
matrix.drawPixel(0,29,oldColor);
drawDot(1);
oldColor = matrix.Color333(0,0,0);
matrix.drawPixel(29,29,oldColor);
}

//*****************************************************
void drawOuterFrame(int red, int green, int blue){

//top/bottom  lines
for (int idy=0;idy<=1;idy++){
  for (int idx=0;idx<=30;idx++)
    {
    cursorY = (idy*30);
    cursorX = idx;
    drawDot(1);
    oldColor = matrix.Color333(red,green,blue);
    matrix.drawPixel(cursorX, cursorY,oldColor);
    }
}

//left/right  lines
for (int idx=0;idx<=1;idx++){
  for (int idy=0;idy<=30;idy++)
    {
    cursorY = idy;
    cursorX = (idx*30);
    drawDot(1);
    oldColor = matrix.Color333(red,green,blue);
    matrix.drawPixel(cursorX, cursorY,oldColor);
    }
}
}

//*************************************************
int decodeBrightPins(){
  
int bright = 0;  
if ((digitalRead(brightOnePin)) == HIGH)(bright = 1);
if ((digitalRead(brightTwoPin)) == HIGH)(bright = bright + 2);
if ((digitalRead(brightFourPin)) == HIGH)(bright = bright + 4);
if ((digitalRead(brightEightPin)) == HIGH)(bright = bright + 8);

return(bright);
}

//*************************************************
//  11111111111111111111111111111111111111111111
//*************************************************
void drawNum1(int XX,int YY){                               //####
  int brightness;
  brightness = decodeBrightPins();
  drawFrame();
  drawBlankSquare(XX,YY,Xmove,Ymove);
  for (int idx=0;idx<=7;idx++)
  {
  cursorX = num1Array[idx][0] + XX + Xmove;     
  cursorY = num1Array[idx][1] + YY + Ymove;     
  drawDot(1);
  if (brightness == 1)(brightnessFlag1 = 1);
    if (brightnessFlag1 == 1)
      {
      oldColor = matrix.Color333(7,0,0);
      }
    else
      {
      oldColor = matrix.Color333(1,0,0);
      }
  matrix.drawPixel(cursorX, cursorY,oldColor);
  }
}

//*************************************************
//  222222222222222222222222222222222222222222222
//*************************************************
void drawNum2(int XX,int YY){ 
  int brightness; 
  brightness = decodeBrightPins();
  drawFrame();
  drawBlankSquare(XX,YY,Xmove,Ymove);
  for (int idx=0;idx<=15;idx++)
   {
   cursorX = num2Array[idx][0] + XX + Xmove;    
   cursorY = num2Array[idx][1] + YY + Ymove;    
   drawDot(1);
   if (brightness == 2)(brightnessFlag2 = 1);
    if (brightnessFlag2 == 1)
      {
      oldColor = matrix.Color333(7,0,0);
      }
    else
      {
      oldColor = matrix.Color333(1,0,0);
      }
   matrix.drawPixel(cursorX, cursorY,oldColor);
  }
}  

//*************************************************
//  333333333333333333333333333333333333333333333
//*************************************************
void drawNum3(int XX,int YY){
  int brightness;
  brightness = decodeBrightPins();
  drawFrame();
  drawBlankSquare(XX,YY,Xmove,Ymove);
  for (int idx=0;idx<=14;idx++)
  {
  cursorX = num3Array[idx][0] + XX + Xmove;
  cursorY = num3Array[idx][1] + YY + Ymove;
  drawDot(1);
  if (brightness == 3)(brightnessFlag3 = 1);
    if (brightnessFlag3 == 1)
      {
      oldColor = matrix.Color333(7,0,0);
      }
    else
      {
      oldColor = matrix.Color333(1,0,0);
      }
  matrix.drawPixel(cursorX, cursorY,oldColor);
  }
}  

//*************************************************
// 4444444444444444444444444444444444444444444444
//*************************************************
void drawNum4(int XX,int YY){
  int brightness;
  brightness = decodeBrightPins();
  drawBlankSquare(XX,YY,Xmove,Ymove);
  for (int idx=0;idx<=12;idx++)
    {
    cursorX = num4Array[idx][0] + XX + Xmove;
    cursorY = num4Array[idx][1] + YY + Ymove;
    drawDot(1);
    if (brightness == 4)(brightnessFlag4 = 1);
    if (brightnessFlag4 == 1)
      {
      oldColor = matrix.Color333(7,0,0);
      }
    else
      {
      oldColor = matrix.Color333(1,0,0);
      }
    matrix.drawPixel(cursorX, cursorY,oldColor);
    }  
}

//*************************************************
//  55555555555555555555555555555555555555555555
//*************************************************
void drawNum5(int XX,int YY){
  int brightness;
  brightness = decodeBrightPins();
  drawBlankSquare(XX,YY,Xmove,Ymove);
  for (int idx=0;idx<=16;idx++)
  {
  cursorX = num5Array[idx][0] + XX + Xmove;
  cursorY = num5Array[idx][1] + YY + Ymove;
  drawDot(1);
  if (brightness == 5)(brightnessFlag5 = 1);
    if (brightnessFlag5 == 1)
      {
      oldColor = matrix.Color333(7,0,0);
      }
    else
      {
      oldColor = matrix.Color333(1,0,0);
      }
  matrix.drawPixel(cursorX, cursorY,oldColor);
  }
}

//*****************************************************
void drawNum6(int XX,int YY){
  int brightness;
  brightness = decodeBrightPins();           
  drawBlankSquare(XX,YY,Xmove,Ymove);
  for (int idx=0;idx<=16;idx++)
  {
  cursorX = num6Array[idx][0] + XX + Xmove;
  cursorY = num6Array[idx][1] + YY + Ymove;
  drawDot(1);
  if (brightness == 6)(brightnessFlag6 = 1);     
    if (brightnessFlag6 == 1)                    
      {                                          
      oldColor = matrix.Color333(5,0,7);
      }
    else
      {
      oldColor = matrix.Color333(1,0,2);
      }                                          
  matrix.drawPixel(cursorX, cursorY,oldColor);
  }
}

//*****************************************************
void drawNum7(int XX,int YY){
  int brightness;
  brightness = decodeBrightPins();
  drawBlankSquare(XX,YY,Xmove,Ymove);
  for (int idx=0;idx<=10;idx++)
  {
  cursorX = num7Array[idx][0] + XX + Xmove;
  cursorY = num7Array[idx][1] + YY + Ymove;
  drawDot(1);
  if (brightness == 7)(brightnessFlag7 = 1);
    if (brightnessFlag7 == 1)
      {
      oldColor = matrix.Color333(5,0,7);
      }
    else
      {
      oldColor = matrix.Color333(1,0,2);
      }
  matrix.drawPixel(cursorX, cursorY,oldColor);
  //delay(50);
  }
}


//****************************************************
void drawNum8(int XX,int YY){
  int brightness;
  brightness = decodeBrightPins(); 
  drawBlankSquare(XX,YY,Xmove,Ymove);
  for (int idx=0;idx<=17;idx++)
  {
  cursorX = num8Array[idx][0] + XX + Xmove;
  cursorY = num8Array[idx][1] + YY + Ymove;
  drawDot(1);
  if (brightness == 8)(brightnessFlag8 = 1);
    if (brightnessFlag8 == 1)
      {
      oldColor = matrix.Color333(5,0,7);
      }
    else
      {
      oldColor = matrix.Color333(1,0,2);
      }
  matrix.drawPixel(cursorX, cursorY,oldColor);
  }  
}

//****************************************************
void drawNum9(int XX,int YY){
  int brightness;
  brightness = decodeBrightPins();   
  drawBlankSquare(XX,YY,Xmove,Ymove);
  for (int idx=0;idx<=16;idx++)
  {
  cursorX = num9Array[idx][0] + XX + Xmove;
  cursorY = num9Array[idx][1] + YY + Ymove;
  drawDot(1);
  if (brightness == 9)(brightnessFlag9 = 1);
    if (brightnessFlag9 == 1)
      {
      oldColor = matrix.Color333(7,0,0);
      }
    else
      {
      oldColor = matrix.Color333(1,0,0);
      }
  matrix.drawPixel(cursorX, cursorY,oldColor);
  }  
}

//*****************************************************
void drawNum10(int XX,int YY){
  int brightness;
  brightness = decodeBrightPins(); 
  drawBlankSquare(XX,YY,Xmove,Ymove);
  for (int idx=0;idx<=16;idx++)
  {
  cursorX = num10Array[idx][0] + XX + Xmove;
  cursorY = num10Array[idx][1] + YY + Ymove;
  drawDot(1);
  if (brightness == 10)(brightnessFlag10 = 1);
    if (brightnessFlag10 == 1)
      {
      //oldColor = matrix.Color333(0,0,7);
      oldColor = matrix.Color333(0,4,4);
      }
    else
      {
      //oldColor = matrix.Color333(0,0,2);
      oldColor = matrix.Color333(0,1,1);
      }
  matrix.drawPixel(cursorX,cursorY,oldColor);
  }
}

//*****************************************************
void drawNum11(int XX,int YY){
  int brightness;
  brightness = decodeBrightPins(); 
  drawBlankSquare(XX,YY,Xmove,Ymove);
  for (int idx=0;idx<=14;idx++)
  {
  cursorX = num11Array[idx][0] + XX + Xmove;
  cursorY = num11Array[idx][1] + YY + Ymove;
  drawDot(1);
  if (brightness == 11)(brightnessFlag11 = 1);
    if (brightnessFlag11 == 1)
      {
      oldColor = matrix.Color333(0,0,7);
      }
    else
      {
      oldColor = matrix.Color333(0,0,2);
      }
  matrix.drawPixel(cursorX, cursorY,oldColor);
  }
}

//*****************************************************
void drawNum12(int XX,int YY){
  int brightness;
  brightness = decodeBrightPins(); 
  drawBlankSquare(XX,YY,Xmove,Ymove);
  for (int idx=0;idx<=18;idx++)
  {
  cursorX = num12Array[idx][0] + XX + Xmove;
  cursorY = num12Array[idx][1] + YY + Ymove;
  drawDot(1);
  if (brightness == 12)(brightnessFlag12 = 1);
    if (brightnessFlag12 == 1)
      {
      oldColor = matrix.Color333(0,0,7);
      }
    else
      {
      oldColor = matrix.Color333(0,0,2);
      }
  matrix.drawPixel(cursorX, cursorY,oldColor);
  }
}


//****************************************************
void drawNum13(int XX,int YY){
  int brightness;
  brightness = decodeBrightPins(); 
  drawBlankSquare(XX,YY,Xmove,Ymove);
  for (int idx=0;idx<=16;idx++)
  {
  cursorX = num13Array[idx][0] + XX + Xmove;
  cursorY = num13Array[idx][1] + YY + Ymove;
  drawDot(1);
  if (brightness == 13)(brightnessFlag13 = 1);
    if (brightnessFlag13 == 1)
      {
      oldColor = matrix.Color333(7,0,0);
      }
    else
      {
      oldColor = matrix.Color333(1,0,0);
      }
  matrix.drawPixel(cursorX, cursorY,oldColor);
  }  
}

//****************************************************
void drawNum14(int XX,int YY){
  int brightness;
  brightness = decodeBrightPins(); 
  drawBlankSquare(XX,YY,Xmove,Ymove);
  for (int idx=0;idx<=17;idx++)
  {
  cursorX = num14Array[idx][0] + XX + Xmove;
  cursorY = num14Array[idx][1] + YY + Ymove;
  drawDot(1);
  if (brightness == 14)(brightnessFlag14 = 1);
    if (brightnessFlag14 == 1)
      {
      oldColor = matrix.Color333(0,4,4);
      }
    else
      {
      oldColor = matrix.Color333(0,1,1);
      }
  matrix.drawPixel(cursorX, cursorY,oldColor);
  }  
}

//*****************************************************
void drawNum15(int XX,int YY){
  int brightness;
  brightness = decodeBrightPins(); 
  drawBlankSquare(XX,YY,Xmove,Ymove);
  for (int idx=0;idx<=19;idx++)
  {
  cursorX = num15Array[idx][0] + XX + Xmove;
  cursorY = num15Array[idx][1] + YY + Ymove;
  drawDot(1);
  if (brightness == 15)(brightnessFlag15 = 1);
    if (brightnessFlag15 == 1)
      {
      oldColor = matrix.Color333(0,0,7);
      }
    else
      {
      oldColor = matrix.Color333(0,0,2);
      }
  matrix.drawPixel(cursorX, cursorY,oldColor);
  }
}

//******************************************************
void drawNum16(int XX,int YY){
   drawBlankSquare(XX,YY,0,0);
}

//******************************************************
void blankEasel()
{
  cursorX = 0;
  cursorY = 0;
  oldColor = 0;
  matrix.fillScreen(0); // Blank screen
}

//******************************************************
// Draw dot places a pixel where the cursorX and cursorY variables
// are.
// Color is determined by the draw boolean parameter. 0 erases a
// pixel (draws blank). 1 draws the active color.
void drawDot(boolean draw)
{
  // We need to both update the oldColor varaiable, which keeps
  // track of a color under the cursor. And send the drawPixel
  // command. First update oldColor:
  if (draw) // if 1 Draw pixel
    oldColor = matrix.Color333(red, green, blue);
  else      // else Erase
    oldColor = 0;
  // Then draw the desired color:
  matrix.drawPixel(cursorX, cursorY, oldColor);
}
//************************************************************

void resetBrightnessFlags(){

brightnessFlag1 = 0;
brightnessFlag2 = 0;
brightnessFlag3 = 0;
brightnessFlag4 = 0;
brightnessFlag5 = 0;
brightnessFlag6 = 0;
brightnessFlag7 = 0;
brightnessFlag8 = 0;
brightnessFlag9 = 0;
brightnessFlag10 = 0;
brightnessFlag11 = 0;
brightnessFlag12 = 0;
brightnessFlag13 = 0;
brightnessFlag14 = 0;
brightnessFlag15 = 0;
}

//end of program, however...
//****************************************************************************

//NOT USED AT PRESENT
void drawBrightNumberFrame(int XX1, int YY1, int Xmove1, int Ymove1){
//draw horiz lines 
for (int idx=0;idx<=7;idx++)
{
  cursorX = idx + XX1+ Xmove1;     //####
  cursorY = 0 + YY1 + Ymove1;     //####
  drawDot(1);
  oldColor = matrix.Color333(0,7,0);
  matrix.drawPixel(cursorX, cursorY,oldColor);  
}
for (int idx=0;idx<=7;idx++)
{
  cursorX = idx + XX1+ Xmove1;     //####
  cursorY = 7 + YY1 + Ymove1;     //####
  drawDot(1);
  oldColor = matrix.Color333(0,7,0);
  matrix.drawPixel(cursorX, cursorY,oldColor);  
}
//draw vert lines
for (int idx=0;idx<=7;idx++)
{
  cursorX = 0 + XX1+ Xmove1;     //####
  cursorY = idx + YY1 + Ymove1;     //####
  drawDot(1);
  oldColor = matrix.Color333(0,7,0);
  matrix.drawPixel(cursorX, cursorY,oldColor);  
}
for (int idx=0;idx<=7;idx++)
{
  cursorX = 7 + XX1+ Xmove1;     //####
  cursorY = idx + YY1 + Ymove1;     //####
  drawDot(1);
  oldColor = matrix.Color333(0,7,0);
  matrix.drawPixel(cursorX, cursorY,oldColor);  
}
}

//********************************************************************************
// SAVE SAVE SAVE ###  FUNCTIONS BELOW ARE NOT USED NOW BUT MAY BE IN THE FUTURE
//********************************************************************************
/*

matrix.setTextColor(matrix.Color333(red, green, blue));
matrix.setCursor(cursorX, cursorY);
matrix.drawLine(shapeX0, shapeY0, cursorX, cursorY,
                    matrix.Color333(red, green, blue));

matrix.fillRect(shapeX0, shapeY0, w, h,
                      matrix.Color333(red, green, blue));
    else
      matrix.drawRect(shapeX0, shapeY0, w, h,
                      matrix.Color333(red, green, blue));

matrix.fillCircle(shapeX0, shapeY0, radius,
                        matrix.Color333(red, green, blue));
    }
    else // Empty circle
    {
      // Draw an empty circle:
      matrix.drawCircle(shapeX0, shapeY0, radius,
                        matrix.Color333(red, green, blue));
      // Draw the old color back in the center:
      matrix.drawPixel(shapeX0, shapeY0, oldCenterColor);

 matrix.setCursor(cursorX, cursorY); // Update cursor
      }
      else // Otherwise it's a printable character
      {
        matrix.print(c); // Print char
        cursorX += 6; // Update our cursor
        if (cursorX >= (matrix.width() - 6))
        { // If x is to far to the right, go down a line:
          cursorX = 0;
          cursorY += 8;
        }
      }
    }
  }
  // Check to make sure y doesn't go off screen:
  if (cursorY >= matrix.height())
    cursorY = matrix.height() - 1;
}

blinkCursor(false); // Turn cursor off first.  

void loadBitmap()
{
  uint8_t *ptr = matrix.backBuffer(); // Get address of matrix data
  memcpy_P(ptr, bmp, sizeof(bmp));

  // update oldColor before the loop() redraws our cursor.
  oldColor = getPixelColor(cursorX, cursorY);
}

*/
//************************************************************************
//       UNUSED PAINT CODE - SAVE
//************************************************************************
  
/*  
// Draw dot places a pixel where the cursorX and cursorY variables
// are.
// Color is determined by the draw boolean parameter. 0 erases a
// pixel (draws blank). 1 draws the active color.
//void drawDot(boolean draw)
{
  // We need to both update the oldColor variable, which keeps
  // track of a color under the cursor. And send the drawPixel
  // command. First update oldColor:
  if (draw) // if 1 Draw pixel
    oldColor = matrix.Color333(red, green, blue);
  else      // else Erase
    oldColor = 0;
  // Then draw the desired color:
  matrix.drawPixel(cursorX, cursorY, oldColor);
}
*/

  /*     
  // These variables keep track of the cursor status. lastUpdate
  // remembers when we last did something to the cursor.
  // cursorStatus remembers if the cursor is on or off.
  static uint32_t lastUpdate = 0;
  static boolean cursorStatus = true;
  // If enough time has passed, let's blink the cursor:
  if (millis() > lastUpdate + BLINK_RATE)
  {
    lastUpdate = millis(); // get a new lastUpdate value
    blinkCursor(cursorStatus); // blinkCursor on or off
    cursorStatus = !cursorStatus; // Update cursorStatus
  }

  // This huge block checks if serial data is available. If so
  // it'll act on that data:
  if (Serial.available())
  {
    char c = Serial.read();     // Read data into c
    switch (c)
    {
      //////////////
      // Movement //
      //////////////
      case 'w':                 // w - Move up
        moveCursor(UP, 1);
        break;
      case 'W':                 // W - move up 2
        moveCursor(UP, 2);
        break;
      case 's':                 // s - Move down
        moveCursor(DOWN, 1);
        break;
      case 'S':                 // S - Move down 2
        moveCursor(DOWN, 2);
        break;
      case 'a':                 // a - Move left
        moveCursor(LEFT, 1);
        break;
      case 'A':                 // A - Move left 2
        moveCursor(LEFT, 2);
        break;
      case 'd':                 // d - Move Right
        moveCursor(RIGHT, 1);
        break;
      case 'D':                 // D - Move right 2
        moveCursor(RIGHT, 2);
        break;
      ///////////////////
      // Color Control //
      ///////////////////
      case 'r': // Red down
        if (red > 0) red--;
        break;
      case 'R': // Red up
        if (red < 7) red++;
        break;
      case 'b': // blue down
        if (blue > 0) blue--;
        break;
      case 'B': // blue up (hehe)
        if (blue < 7) blue++;
        break;
      case 'g': // green down
        if (green > 0) green--;
        break;
      case 'G': // green up
        if (green < 7) green++;
        break;
      case 'z': // Copy color
        copyColor();
      /////////////////////////
      // Draw / Erase Pixels //
      /////////////////////////
      case ' ':                 // Space - Place pixel
        drawDot(true);
        break;
      case 'e':                 // e - Erase pixel
        drawDot(false);
        break;
      /////////////////
      // Draw Shapes //
      /////////////////
      case 'v':                 // v - draw a vector (line)
        drawLine();
        break;
      case 'x':                 // x - start box / draw box
        drawBox(false);
        break;
      case 'X':                 // X - start box / fill box
        drawBox(true);
        break;
      case 'c':                 // c - start circle / draw circle
        drawCircle(false);
        break;
      case 'C':                 // C - start circle / fill circle
        drawCircle(true);
        break;
      case 't':                 // t - Enter text mode
        drawText();
        break;
      case 'F':
      case 'f':                 // f - Fill screen with active color
        matrix.fillScreen(matrix.Color333(red, green, blue));
        drawDot(true);
        break;
      case 'E':                 // E - Erase screen, reset cursor
        blankEasel();
        break;
      case 'p':
        printBuffer();
        break;
      case 'l':
        loadBitmap();
        break;
    }
  }
//}
*/

/*
// drawLine should be called twice to draw a line. The first
// time it's called (if activeDraw is not LINE), it'll set up the
// global variables it needs. The second time it's called, it
// will actually draw the line.
void drawLine()
{
  // activeDraw is a global variable which keeps track of which
  // (if any) shape drawing mode we're in.
  if (activeDraw != LINE) // If we're not already in LINE mode:
  {
    drawDot(true);  // Place a pixel where the cursor is
    shapeX0 = cursorX;  // Update shapeX0 variable with current x
    shapeY0 = cursorY;  // Update shapeY0 variable with current y
    activeDraw = LINE;  // Next time drawLine is called we'll draw
  }
  else // Otherwise, we're in line mode and we already have
  { // the first point
    // Draw a line from previously stored variables to the
    // current cursor position:
    matrix.drawLine(shapeX0, shapeY0, cursorX, cursorY,
                    matrix.Color333(red, green, blue));
    drawDot(true);  // Draw a dot (update oldColor variable)
    activeDraw = NONE; // Revert to no shape-mode active
  }
}

// drawBox should be called twice to draw a box. First to set the
// first corner. Second to set the diagonally opposite corner,
// and actually draw the box.
void drawBox(boolean fill)
{
  if (activeDraw != RECT) // If we're not already in RECT mode:
  {
    drawDot(true);  // Place a pixel where the cursor is
    shapeX0 = cursorX; // Set shapeX0 to current x position
    shapeY0 = cursorY; // Set shape Y0 to current y position
    activeDraw = RECT; // Next time this is called, we'll draw
  }
  else // Otherwise, we can draw a rectangle:
  {
    // Need to calculate the width and height of the box, based
    // on previously stored values and current cursor position:
    int16_t w = abs(cursorX - shapeX0) + 1;
    int16_t h = abs(cursorY - shapeY0) + 1;
    // drawRect draws from top-left corner, find that by getting
    // the minimum coordinate of each corner:
    shapeX0 = min(shapeX0, cursorX);
    shapeY0 = min(shapeY0, cursorY);
    // Use all the values we calculated to draw a box:
    if (fill)
      matrix.fillRect(shapeX0, shapeY0, w, h,
                      matrix.Color333(red, green, blue));
    else
      matrix.drawRect(shapeX0, shapeY0, w, h,
                      matrix.Color333(red, green, blue));
    drawDot(true);  // Draw a pixel (just  to update oldColor)
    activeDraw = NONE; // Back to no active shape mode
  }
}

// drawCircle should be called twice to draw a circle. Call first
// to set the position of the center. Call again to set the
// outer radius value.
void drawCircle(boolean fill)
{
  if (activeDraw != CIRCLE) // If we're not already in CIRCLE mode
  {
    // Let's place a pixel so we can visually see the center.
    // We may need to remove the pixel (if not filling) so store
    // the old pixel color first.
    oldCenterColor = oldColor;
    drawDot(true);  // Place pixel where cursor is
    shapeX0 = cursorX; // Store x and y so we remember where
    shapeY0 = cursorY; // the center is
    activeDraw = CIRCLE; // Next time we'll draw a circle:
  }
  else
  {
    // We'll need to calculate the radius of the circle. Time
    // to whip out some pythagorean theorum!
    // First, find the x and y distances from center:
    uint8_t x = abs(cursorX - shapeX0);
    uint8_t y = abs(cursorY - shapeY0);
    // Then r^2 = x^2 + y^2, solve for r
    int16_t radius = (int16_t) sqrt(x * x + y * y);
    // Now we can draw our circle:
    if (fill)
    {
      // Draw a filled circle:
      matrix.fillCircle(shapeX0, shapeY0, radius,
                        matrix.Color333(red, green, blue));
    }
    else // Empty circle
    {
      // Draw an empty circle:
      matrix.drawCircle(shapeX0, shapeY0, radius,
                        matrix.Color333(red, green, blue));
      // Draw the old color back in the center:
      matrix.drawPixel(shapeX0, shapeY0, oldCenterColor);
    }
    drawDot(true); // Draw pixel at cursor, update oldColor
    activeDraw = NONE; // Revert back to no active shape mode
  }
}

// drawText is used to draw text on the display. It'll enter a
// loop, continuously receiving serial chars and printing them.
// It'll exit when EXIT_CHAR is received over serial.
void drawText()
{
  char c;
  blinkCursor(0); // Turn cursor off

  // Setup text color and cursor:
  matrix.setTextColor(matrix.Color333(red, green, blue));
  matrix.setCursor(cursorX, cursorY);

  // Print characters until receive EXIT_CHAR (`)
  while (c != EXIT_CHAR)
  {
    while (!Serial.available()) ; // loop and wait for serial
    c = Serial.read();  // read serial data into c
    if (c != EXIT_CHAR)
    {
      if (c == '\n')   // CTRL+ENTER = newline / linefeed
      {
        cursorY += 8; // Increase another line
        cursorX = 0;  // Back to beginning
        matrix.setCursor(cursorX, cursorY); // Update cursor
      }
      else if (c == '\r') // ENTER
      {
        cursorY += 8; // Increase another line
        matrix.setCursor(cursorX, cursorY); // Update cursor
      }
      else // Otherwise it's a printable character
      {
        matrix.print(c); // Print char
        cursorX += 6; // Update our cursor
        if (cursorX >= (matrix.width() - 6))
        { // If x is to far to the right, go down a line:
          cursorX = 0;
          cursorY += 8;
        }
      }
    }
  }
  // Check to make sure y doesn't go off screen:
  if (cursorY >= matrix.height())
    cursorY = matrix.height() - 1;
}

// moveCursor moves the cursorX and cursorY variables UP, DOWN,
// LEFT, or RIGHT in distance number of spaces. It'll wrap around
// the screen if moved out of bounds.
// This function also keeps track of any old pixels under the
// cursor. It'll redraw them when the cursor moves.
void moveCursor(uint8_t direction, uint8_t distance)
{
  // If there was a color under the cursor, put it back:
  matrix.drawPixel(cursorX, cursorY, oldColor);

  // For each direction, decrease it by distance, then check
  // to make sure it's not out of bounds. If it is, wrap around
  // to the other side.
  switch (direction)
  {
    case UP:
      cursorY -= distance;
      if (cursorY < 0) cursorY = matrix.height() - 1;
      break;
    case DOWN:
      cursorY += distance;
      if (cursorY > matrix.height() - 1) cursorY = 0;
      break;
    case LEFT:
      cursorX -= distance;
      if (cursorX < 0) cursorX = matrix.width() - 1;
      break;
    case RIGHT:
      cursorX += distance;
      if (cursorX > matrix.width() - 1) cursorX = 0;
      break;
  }
  // update oldColor before the loop() redraws our cursor.
  oldColor = getPixelColor(cursorX, cursorY);
}

// blinkCursor turns the cursor either on or off. Cursor color
// is determined by red, green, and blue variable values.
void blinkCursor(boolean stat)
{
  if (stat) // Blink on:
    matrix.drawPixel(cursorX, cursorY, matrix.Color333(red, green, blue));
  else      // Blink off
    matrix.drawPixel(cursorX, cursorY, 0);
}

// This is a slightly modified version of the dumpMatrix function
// from the RGBmatrixPanel library. Running out of SRAM! So some
// of the static serial prints are stored in flash ("F()"). Also
// do some extra formatting so it works with non-Serial monitor
// terminals.
void printBuffer()
{
  blinkCursor(false); // Turn cursor off first.

  int bufferSize = matrix.width() * (matrix.height() / 2) * 3;
  // Get a pointer to the data buffer:
  uint8_t * buf = matrix.backBuffer();

  // Print some stuff
  Serial.println();
  Serial.print(F("#include <avr/pgmspace.h>\r\n\r\n"));
  Serial.print(F("static const uint8_t PROGMEM bmp[] = {\r\n"));

  for (int i = 0; i < bufferSize; i++)
  {
    Serial.print(F("0x"));
    if (buf[i] < 0x10)
      Serial.print('0');
    Serial.print(buf[i], HEX);
    if (i < (bufferSize - 1))
    {
      if ((i & 7) == 7)
        Serial.print(F(", \r\n "));
      else
        Serial.print(',');
    }
  }
  Serial.println(F("\r\n};"));
}

// Copy color updates the red, green, and blue variables to match
// the oldColor variable values.
void copyColor()
{
  red = (oldColor & 0xE000) >> 13;
  green = (oldColor & 0x0700) >> 8;
  blue = (oldColor & 0x001C) >> 2;
}

// loadBitmap loads up a stored bitmap into the matrix's data
// buffer. If you've printed an image out, use this to load it
// back onto the screen.
void loadBitmap()
{
  uint8_t *ptr = matrix.backBuffer(); // Get address of matrix data
  memcpy_P(ptr, bmp, sizeof(bmp));

  // update oldColor before the loop() redraws our cursor.
  oldColor = getPixelColor(cursorX, cursorY);
}

// Reset the screen. Set cursors back to top-left. Reset oldColor
// and blank the screen.
void blankEasel()
{
  cursorX = 0;
  cursorY = 0;
  oldColor = 0;
  matrix.fillScreen(0); // Blank screen
}

// getPixelColor digs into the rgbMatrixPanel library, and the
// buffer where it stores data, to retreive the color of a pixel.
// Most of this was convoluted from the drawPixel function in the
// rgbMatrixPanel library.
// This function is called in the moveCursor function, to restore
// a pixel color if the cursor is on top of it.
uint16_t getPixelColor(uint8_t x, uint8_t y)
{
  uint8_t * ptr; // A movable pointer to read data from
  uint16_t r = 0, g = 0, b = 0; // We'll store rgb values in these
  uint8_t nPlanes = 4; // 4 planes. This is static in the library
  uint8_t nRows = matrix.height() / 2; // 16 for 32 row, 8 for 16

  // Get a pointer to the matrix backBuffer, where the pixel/
  // color data is stored:
  uint8_t * buf = matrix.backBuffer();

  // Data for upper half is stored in lower bits of each byte
  if (y < nRows)
  {
    ptr = &buf[y * matrix.width() * (nPlanes - 1) + x];
    // get data from plane 0:
    if (ptr[64] & 0x01)
      r = 1;
    if (ptr[64] & 0x02)
      g = 1;
    if (ptr[32] & 0x01)
      b = 1;
    // get data from other planes:
    for (uint8_t i = 2; i < (1 << nPlanes);  i <<= 1)
    {
      if (*ptr & 0x04)
        r |= i;
      if (*ptr & 0x08)
        g |= i;
      if (*ptr & 0x10)
        b |= i;
      ptr += matrix.width();
    }
  }
  else
  {
    ptr = &buf[(y - nRows) * matrix.width() * (nPlanes - 1) + x];
    // get data from plane 0
    if (ptr[32] & 0x02)
      r = 1;
    if (*ptr & 0x01)
      g = 1;
    if (*ptr & 0x02)
      b = 1;
    for (uint8_t i = 2; i < (1 << nPlanes); i <<= 1)
    {
      if (*ptr & 0x20)
        r |= i;
      if (*ptr & 0x40)
        g |= i;
      if (*ptr & 0x80)
        b |= i;
      ptr += matrix.width();
    }
  }
  return (r << 12) | (g << 7) | (b << 1);
}
*/
