/* ######################
 *  DOBOT CONTROL SKETCH 
 * ######################
 * 
 * Project Title: Puzzle-Bot
 * Purpose: Use a Dobot Magician robot arm to solve a 4x4 sliding puzzle
 * Microcontroller: Arduino Mega 2560
 * 
 * File: DobotController14 
 * Date: 8/17/23
 * 
 * Written by: Dave Goodsell & Dobot
 * dsgoodsell@verizon.net
 * 
 */
  
// Ver.9:  Calibrating iniX and iniY to actual puzzle location. DONE. 
// Ver.10: Stuff.
// Ver.11: More stuff.      
// Ver.12: Code cleanup
// Ver.13: Doing full system integration with Matt's camera
// Ver.14: Line 793, adding Elapsed Dobot moving time to LCD display, too many issues, deleted
 
// COM9    COM9   COM9   

//The modularized code was written by the Dobot people. 
//I did not modify any of their modules.

/****************************************Copyright(c)*****************************************************
**                            Shenzhen Yuejiang Technology Co., LTD.
**
**                                 http://www.dobot.cc
**
**--------------File Info---------------------------------------------------------------------------------
** File name:           main.cpp
** Latest modified Date:2016-10-24
** Latest Version:      V2.0.0
** Descriptions:        main body
**
**--------------------------------------------------------------------------------------------------------
** Modify by:           Edward
** Modified date:       2016-11-25
** Version:             V1.0.0
** Descriptions:        Modified,From DobotDemoForSTM32
**--------------------------------------------------------------------------------------------------------
*********************************************************************************************************/
#include "stdio.h"
#include "Protocol.h"
#include "command.h"
#include "FlexiTimer2.h"
#include <EEPROM.h>

//Set Serial TX&RX Buffer Size
#define SERIAL_TX_BUFFER_SIZE 64
#define SERIAL_RX_BUFFER_SIZE 256

//#define JOG_STICK 
/*********************************************************************************************************
       Global parameters
*********************************************************************************************************/
EndEffectorParams gEndEffectorParams;

JOGJointParams  gJOGJointParams;
JOGCoordinateParams gJOGCoordinateParams;
JOGCommonParams gJOGCommonParams;
JOGCmd          gJOGCmd;

PTPCoordinateParams gPTPCoordinateParams;
PTPCommonParams gPTPCommonParams;
PTPCmd          gPTPCmd;

uint64_t gQueuedCmdIndex;

//[0][0] = mode type(1=MOVJ,2=MOVL)    //MOVL is a straight line move, MOVJ is an arc
//[0][1-4] = x,y,z,r                   //r is rotation when a servo is mounted on the arm
//[0][5] = pause
//[0][6] = suction (1=on)              //there is a suction cup accessory
//Home = 1,200,0,0,0,0,0

float iniX = 217.2;    //initial X position of stylus on center of Position 1 of puzzle 
float iniY =  -19.5;   //initial Y position of stylus on center of Position 1 of puzzle
float iniZ = 0;

int idx;    
float newPos; 
float newPos1;

float x4 = 41;         //This important number defines the horiz center of Position 16 of the puzzle
float x3 = (x4 * 2/3);
float x2 = (x4 * 1/3);
float x1 = 0;

float y4 = 42;         //This number important defines the vert center of Position 16 of the puzzle
float y3 = (y4 * 2/3);
float y2 = (y4 * 1/3);
float y1 = 0;

float extra = 1.2;

float upZ = -19;     //adjusted by upZ pot, as req'd and stored in EEPROM  
float dwnZ = -23;    //DEFAULT = -23 OR SET BY POT AT ANY TIME, USING TOGGLE SWITCH AND RESET 

float absX;   //final X to do ProcessProtocol();
float absY;   //etc

int dly = 10;
int receivingBuffer[100];
int x;

int onePinHigh = 45;  //not used any longer, all of the below pins, 44 to 51
int onePinLow = 44;
int twoPinHigh = 47;
int twoPinLow = 46;
int fourPinHigh = 49;
int fourPinLow = 48;
int eightPinHigh = 51;
int eightPinLow = 50;

int adjustStylusDownPosSw = 33; 
int blinkBluPin = 43;
int blinkBluLowPin = 41;
int blinkGrnPin = 39;
int blinkGrnLowPin = 38;
int blinkYelPin = 35;
int blinkYelLowPin = 34;
int okayToSendMoveToDisplayNow = 27;

//A0-A7 self-named
int CTSToSolve = A8;
int DataReadyFromSolve = A9;
int ResetFromSolve = A10;

int potInput = A15;
float downAdjustmentRange = 8;  //approx 8mm
float upAdjustmentRange = 8;
float XcenterOfPuzzle = 237;
float displacementSensorPin = A13;   
int displacementCount;
int upZPotPin = A12;
int adjustStylusUpZSw = 2;
int upZPotValue;

int From;
int To;
int Move = 0;
long dobotStartTime; 

int startButton = 53;
int displacementSensorEnablePin = 9;  //on = low


/*********************************************************************************************************
** Function name:       setup
** Descriptions:        Initializes Serial
** Input parameters:    none
** Output parameters:   none
** Returned value:      none
*********************************************************************************************************/
void setup() {
    Serial.begin(115200);
    Serial1.begin(115200); 
    Serial2.begin(9600);    //2-line LCD
    Serial3.begin(115200);
    printf_begin();
    //Set Timer Interrupt
    FlexiTimer2::set(100,Serialread); 
    FlexiTimer2::start();

    pinMode(onePinHigh, OUTPUT); 
    pinMode(onePinLow, OUTPUT);
    pinMode(twoPinHigh, OUTPUT); 
    pinMode(twoPinLow, OUTPUT);
    pinMode(fourPinHigh, OUTPUT); 
    pinMode(fourPinLow, OUTPUT);
    pinMode(eightPinHigh, OUTPUT); 
    pinMode(eightPinLow, OUTPUT);

    pinMode(blinkBluPin, OUTPUT);
    pinMode(blinkBluLowPin, OUTPUT);
    pinMode(blinkGrnPin, OUTPUT);
    pinMode(blinkGrnLowPin, OUTPUT);
    pinMode(blinkYelPin, OUTPUT);
    pinMode(blinkYelLowPin, OUTPUT);

    digitalWrite(onePinLow, LOW);
    digitalWrite(twoPinLow, LOW);
    digitalWrite(fourPinLow, LOW);
    digitalWrite(eightPinLow, LOW);

    digitalWrite(blinkBluLowPin, LOW);
    digitalWrite(blinkGrnLowPin, LOW);
    digitalWrite(blinkYelLowPin, LOW);
    
    pinMode(startButton, INPUT_PULLUP);
    pinMode(adjustStylusDownPosSw, INPUT_PULLUP);
    pinMode(displacementSensorPin, INPUT);  
    pinMode(displacementSensorEnablePin, INPUT_PULLUP);
    pinMode(A0, INPUT); 
    pinMode(A1, INPUT);
    pinMode(A2, INPUT);
    pinMode(A3, INPUT);
    pinMode(A4, INPUT);
    pinMode(A5, INPUT);
    pinMode(A6, INPUT);
    pinMode(A7, INPUT);
    pinMode(CTSToSolve, OUTPUT);     //A8
    pinMode(DataReadyFromSolve, INPUT);  //A9
    pinMode(ResetFromSolve, INPUT);      //A10
    pinMode(okayToSendMoveToDisplayNow, INPUT);  // pin 27 
    pinMode(potInput, INPUT);
    pinMode(upZPotPin, INPUT);
    pinMode(adjustStylusUpZSw, INPUT_PULLUP);

    Serial2.write(254);  //turn off cursor
    Serial2.write(12);
    Serial2.write(254);  //clear screen
    Serial2.write(1);
    delay(100);

//read dwnZ and upZ stored values in EEPROM
//*****************************************************
    //note: upZ is stored as a point(ex: 19), dwnZ value1 is stored as an offset(ex: 4)   REMEMBER ALL Z'S ARE NEGATIVE IN FINAL FORM
    //typ upZ = -19.00
    //typ dwnZ = -23.00;    //default
    //note: 8.1 is stored in EEPROM as 81, then divided below to give more resolution of 8.1, instead of just Integers
    float storedDwn = EEPROM.read(2);   //note: EEPROM can only store positive values, 0 to 255, so 81 = 8.1 float
    float storedUp = EEPROM.read(3);  //typ: +19
    upZ = -storedUp;
    storedDwn = (storedDwn / 10);
    dwnZ = (upZ + -storedDwn);   //storedUp is typically -19
    //Serial2.write(254);   //save this code
    //Serial2.write(1);
    //delay(1000); 
    //Serial2.print("dwnZ = ");
    //Serial2.print(dwnZ,1);
    //Serial2.write(254);
    //Serial2.write(192);
    //Serial2.write("upZ  = ");
    //Serial2.print(upZ,1);
    //delay(4000);
    
    Serial2.write(254);  //clear screen
    Serial2.write(1);
    delay(15);
    Serial2.print("** PUZZLE BOT **");
    Serial2.write(254);  //move to line2
    Serial2.write(192);
    Serial2.print("Enter Puzzle 1-5");
}

/*********************************************************************************************************
** Function name:       Serialread
** Descriptions:        import data to rxbuffer
** Input parametersnone:
** Output parameters:   
** Returned value:      
*********************************************************************************************************/
void Serialread()
{
  while(Serial1.available()) {
        uint8_t data = Serial1.read();
        if (RingBufferIsFull(&gSerialProtocolHandler.rxRawByteQueue) == false) {
            RingBufferEnqueue(&gSerialProtocolHandler.rxRawByteQueue, &data);
        }
  }
}
/*********************************************************************************************************
** Function name:       Serial_putc
** Descriptions:        Remap Serial to Printf
** Input parametersnone:
** Output parameters:   
** Returned value:      
*********************************************************************************************************/
int Serial_putc( char c, struct __file * )
{
    Serial.write( c );
    return c;
}

/*********************************************************************************************************
** Function name:       printf_begin
** Descriptions:        Initializes Printf
** Input parameters:    
** Output parameters:
** Returned value:      
*********************************************************************************************************/
void printf_begin(void)
{
    fdevopen( &Serial_putc, 0 );
}

/*********************************************************************************************************
** Function name:       InitRAM
** Descriptions:        Initializes a global variable
** Input parameters:    none
** Output parameters:   none
** Returned value:      none
*********************************************************************************************************/
void InitRAM(void)
{
    //Set JOG Model
    gJOGJointParams.velocity[0] = 100;
    gJOGJointParams.velocity[1] = 100;
    gJOGJointParams.velocity[2] = 100;
    gJOGJointParams.velocity[3] = 100;
    gJOGJointParams.acceleration[0] = 80;
    gJOGJointParams.acceleration[1] = 80;
    gJOGJointParams.acceleration[2] = 80;
    gJOGJointParams.acceleration[3] = 80;

    gJOGCoordinateParams.velocity[0] = 100;
    gJOGCoordinateParams.velocity[1] = 100;
    gJOGCoordinateParams.velocity[2] = 100;
    gJOGCoordinateParams.velocity[3] = 100;
    gJOGCoordinateParams.acceleration[0] = 80;
    gJOGCoordinateParams.acceleration[1] = 80;
    gJOGCoordinateParams.acceleration[2] = 80;
    gJOGCoordinateParams.acceleration[3] = 80;

    gJOGCommonParams.velocityRatio = 50;
    gJOGCommonParams.accelerationRatio = 50;
   
    gJOGCmd.cmd = AP_DOWN;
    gJOGCmd.isJoint = JOINT_MODEL;

    

    //Set PTP Model
    gPTPCoordinateParams.xyzVelocity = 100;
    gPTPCoordinateParams.rVelocity = 100;
    gPTPCoordinateParams.xyzAcceleration = 80;
    gPTPCoordinateParams.rAcceleration = 80;

    gPTPCommonParams.velocityRatio = 50;
    gPTPCommonParams.accelerationRatio = 50;


    //Set END EFFECTOR Model
    //gEndEffectorParams.xxxx = xx;  

   
   // DOBOT DEMO INITIAL PARAMS
   gPTPCmd.ptpMode = MOVL_XYZ;
   gPTPCmd.x = 175;   // starting at 200mm out from Dobot axis Centerline(CL)
   gPTPCmd.y = 0;     // side-to-side = straight out front from CL
   gPTPCmd.z = 0;     // up-down = ???
   gPTPCmd.r = 0;     // servo rotation = ???? 

   gQueuedCmdIndex = 0;
    
}

//*************************************************************************************************************************
//*************************************************************************************************************************
//                                            MY FUNCTIONS
//*************************************************************************************************************************
//*************************************************************************************************************************
void latchInMove(){

//Dobot requests a Move 
digitalWrite(CTSToSolve, HIGH);  //A8
BluOn();

//wait for Solve to put Move on pins and signal that it is ready
do
{}
while ((digitalRead(DataReadyFromSolve)) == LOW);  //A9
GrnOn();

//when DataReady goes HIGH, read to pins and translate them to From and To
 From = 0;
  if ((digitalRead(A0))  == 1)
  {(From = 1);}
  if ((digitalRead(A1))  == 1){(From = From + 2);}
  if ((digitalRead(A2))  == 1){(From = From + 4);}
  if ((digitalRead(A3))  == 1){(From = From + 8);}
  if (From == 0){(From = 16);}
 To = 0;
  if ((digitalRead(A4))  == 1)(To = 1);
  if ((digitalRead(A5))  == 1)(To = To + 2);
  if ((digitalRead(A6))  == 1)(To = To + 4);
  if ((digitalRead(A7))  == 1)(To = To + 8);
  if (To == 0)(To = 16);

GrnOff();
 
//tell Solve that the data was received    
digitalWrite(CTSToSolve, LOW);
BluOff();

Serial2.write(254);
Serial2.write(2);
Serial2.print("Moves: ");
if (Move <= 9)
  {
  Serial2.print("0");
  }
Serial2.print(Move);

delay(100);
Move++;

//next step is to do the actual Move
}

//Calculate and add extra push to piece so it presses solidly against next number or frame 
//*****************************************************************************************
void calculateAndAddExtraPush(int from2, int to2){       //typ extra = +-2
  switch (from2 - to2)
   {
    case -1:
     {
     //add extra +2 to absPosY 
     absY = (absY + extra);
     break;
     }

    case 1: 
     {
     //add extra -2 to absPosY 
     absY = (absY - extra);
     break;
     }  

    case -4:
     {
     //add extra +2 to absPosX 
     absX = (absX + extra);
     break;
     }

   case 4:
     {
     //add extra -2 to absPosX
     absX = (absX - extra);
     break;
     }  
   }
}

//lookup table for ABS positions for a jump(MOVJ=1))   MOVL = 1 = DwnZ or a slide
//********************************************************************************
void determineABSPosition(int absPos)
{
switch(absPos)
{
 case 1:
  {
   absX = iniX + x1;
   absY = iniY + y1;
   break;
  }

 case 2:
  {
   absX = iniX + x1;
   absY = iniY + y2;
   break;
  }

 case 3:
  {
   absX = iniX + x1;
   absY = iniY + y3;
   break;
  }

 case 4:
  {
   absX = iniX + x1;
   absY = iniY + y4;
   break;
  }
case 5:
  {
   absX = iniX + x2;
   absY = iniY + y1;
   break;
  }

 case 6:
  {
   absX = iniX + x2;
   absY = iniY + y2;
   break;
  }

 case 7:
  {
   absX = iniX + x2;
   absY = iniY + y3;
   break;
  }

 case 8:
  {
   absX = iniX + x2;
   absY = iniY + y4;
   break;
  }
  
 case 9:
  {
   absX = iniX + x3;
   absY = iniY + y1;
   break;
  }

 case 10:
  {
   absX = iniX + x3;
   absY = iniY + y2;
   break;
  }

 case 11:
  {
   absX = iniX + x3;
   absY = iniY + y3;
   break;
  }

 case 12:
  {
   absX = iniX + x3;
   absY = iniY + y4;
   break;
  }
case 13:
  {
   absX = iniX + x4;
   absY = iniY + y1;
   break;
  }

 case 14:
  {
   absX = iniX + x4;
   absY = iniY + y2;
   break;
  }

 case 15:
  {
   absX = iniX + x4;
   absY = iniY + y3;
   break;
  }

 case 16:
  {
   absX = iniX + x4;
   absY = iniY + y4;
   break;
  }
}
}

/*********************************************************************************************************
** Function name:       loop
** Descriptions:        Program entry
** Input parameters:    none
** Output parameters:   none
** Returned value:      none
*********************************************************************************************************/

//*****************************************************************************************************************
//*****************************************************************************************************************
//*****************************************************************************************************************
//                                  START OF PROGRAM
//*****************************************************************************************************************
//*****************************************************************************************************************
//*****************************************************************************************************************

void loop() 
{
    InitRAM();   //load type,x,y,z,r,pause,suction params

    ProtocolInit();
    
    SetJOGJointParams(&gJOGJointParams, true, &gQueuedCmdIndex);
    
    SetJOGCoordinateParams(&gJOGCoordinateParams, true, &gQueuedCmdIndex);
    
    SetJOGCommonParams(&gJOGCommonParams, true, &gQueuedCmdIndex);
    
    printf("\r\n======Enter demo application======\r\n");
       
    SetPTPCmd(&gPTPCmd, true, &gQueuedCmdIndex);

    ProtocolProcess();

      
 for(; ;)
 {
    
//*******************************************************************************
//       NEW STUFF, CO-MINGLED WITH ORIGINAL DOBOT CODE
//*******************************************************************************

Move = 0;

if ((digitalRead(adjustStylusUpZSw)) == HIGH)(adjustStylusUpPosition());
if ((digitalRead(adjustStylusDownPosSw)) == LOW)(adjustStylusDownPosition());

//definePuzzleCorners();   //this works fine, but it gets in the way

//sub-loop to send multiple Moves to Dobot until receiving a Move[17][17]
sendMovesToDobot();   //see routine below

return;

//*******************************************************************************
//       END OF ALL NEW STUFF
//*******************************************************************************

//BIG NOTE: STOPPING HERE IS IMPORTANT. 
//BUT IF IT GOES TO THE END OF THE LOOP, THERE ARE SCREWY MOVES?????
do   
{}
while(1);

} //end for(; ;)

do
{}
while(1);
    
} //end main loop
//**********************************************************************************************************************


//*******************************************************************************************
//*******************************************************************************************
//             SEND MOVES TO DOBOT, THE MAJOR ROUTINE OF THIS PROGRAM
//*******************************************************************************************
//*******************************************************************************************

void sendMovesToDobot(){

//adjustStylusDownPosition();

//dobotStartTime = millis();  //start of movements

do
{ 
latchInMove();
if (From == 18)(resetNumOfMovesOnLCD());   
if (From == 17) return;   //catches MV,17,17 
if (From == To) return;   //catches 0==0 or 1==1

//*****************************
// FIRST POINT TO SLIDE "FROM"
//*****************************
newPos = From;

//Serial.print("From: " );
//Serial.println(newPos);

//calculate absX & absY
//*********************
determineABSPosition(newPos);   //returns absX,absY 

//fast move to position
//***********************
//load MOVJ,absX,absY,upZ(0)
gPTPCmd.ptpMode = MOVJ_XYZ;
gPTPCmd.x = absX;
gPTPCmd.y = absY;
gPTPCmd.z = upZ;
gPTPCmd.r = 0;
SetPTPCmd(&gPTPCmd, true, &gQueuedCmdIndex);
ProtocolProcess();

delay(dly);

//***********************************************************************************************************
//LOWER STYLUS THEN CHECK IF IT IS PRESSING ON NUMBER, OR HAS FALLEN INTO A DITCH
//***********************************************************************************************************
//ALSO, THIS FUNCTION CAN BE TURNED OFF DURING TESTING SO THE STYLUS CAN PASS OVER THE PUZZLE, NOT CONTACT IT
//***********************************************************************************************************
if ((digitalRead(displacementSensorEnablePin)) == LOW)  //LOW = "RUN SOLVING" with sensor ON to stop it if necessary
{
  //slow lower stylus
  //****************
  //load MOVL,absX,absY,dwnZ(1)
  gPTPCmd.ptpMode = MOVL_XYZ;
  gPTPCmd.x = absX;
  gPTPCmd.y = absY;
  gPTPCmd.z = dwnZ;
  gPTPCmd.r = 0;
  SetPTPCmd(&gPTPCmd, true, &gQueuedCmdIndex);
  ProtocolProcess();

  delay(2000);  //MUST GIVE THE STYLUS A CHANCE TO GO DOWN BEFORE THE POSITION SENSOR MEASURES THE DOWN MOVEMENT.
                //ALTHOUGH, THERE IS SOMETHING FUNNY HERE????? THE SENSOR READING IS TOO SLOW FOR SOME REASON?
                
  //check that stylus has not fallen into a ditch
  //*********************************************
  displacementCount = analogRead(displacementSensorPin);  //425 = dwnZ, set limit at 250, IF <250, STOP!!!!
  Serial2.write(254);
  Serial2.write(1);
  Serial2.print(dwnZ);
  Serial2.print("  ");
  Serial2.print(displacementCount);
  //wait();
 
  if (displacementCount <= 250)
    {
    //raise stylus 
    gPTPCmd.ptpMode = MOVL_XYZ;
    gPTPCmd.x = absX;
    gPTPCmd.y = absY;
    gPTPCmd.z = upZ;
    gPTPCmd.r = 0;
    SetPTPCmd(&gPTPCmd, true, &gQueuedCmdIndex);
    ProtocolProcess();
    haltMessage();     //flashing halt message is displayed, program is halted, waiting for a reboot
    } 
}
//****************************
//SECOND POINT TO SLIDE "TO"
//****************************
newPos1 = To;

//Serial.print(idx);
//Serial.print(",");
//Serial.println(newPos);
//Serial.print(",");

//calculate ABS x & y then add a tiny extra push in the correct direction, using relationship between 1st & 2nd points
//**********************************************************************************************************************
determineABSPosition(newPos1);   //returns absX,absY 
calculateAndAddExtraPush(newPos,newPos1);     //returns absX,absY, but one with extra push

if ((digitalRead(displacementSensorEnablePin)) == LOW) //LOW = "RUN SOLVING" with sensor ON to stop it if necessary
{ 
//slow slide to extra push position
//**********************************
//load MOVL,absX,absY,dwnZ(1)   //slow slide with Z still down
gPTPCmd.ptpMode = MOVL_XYZ;
gPTPCmd.x = absX;
gPTPCmd.y = absY;
gPTPCmd.z = dwnZ;  //KEEP STYLUS LOWERED
gPTPCmd.r = 0;
SetPTPCmd(&gPTPCmd, true, &gQueuedCmdIndex);
ProtocolProcess();
}
else
{
//slow slide to extra push position, BUT NOT LOWERING STYLUS BECAUSE THIS IS JUST A TEST
//**********************************
//load MOVL,absX,absY,dwnZ(1)   //slow slide with Z still down
gPTPCmd.ptpMode = MOVL_XYZ;
gPTPCmd.x = absX;
gPTPCmd.y = absY;
gPTPCmd.z = upZ;   //KEEP STYLUS RAISED
gPTPCmd.r = 0;
SetPTPCmd(&gPTPCmd, true, &gQueuedCmdIndex);
ProtocolProcess();  
}

delay(dly);

//slow raise rod
//***************
//load MOVL,absX,absY,upZ
gPTPCmd.ptpMode = MOVL_XYZ;
gPTPCmd.x = absX;
gPTPCmd.y = absY;
gPTPCmd.z = upZ;
gPTPCmd.r = 0;
SetPTPCmd(&gPTPCmd, true, &gQueuedCmdIndex);
ProtocolProcess();
delay(dly);

//**********************************************************
//send okay to send Move to Display and continue
//**********************************************************
digitalWrite(okayToSendMoveToDisplayNow, HIGH);   //pin 27 handshake with Solve Arduino
delay(10);
digitalWrite(okayToSendMoveToDisplayNow, LOW);

}
while(1);


}
//end of sendMovesToDobot


//*******************************************************************************************
//               OTHER FUNCTIONS
//*******************************************************************************************

void definePuzzleCorners(){    //actually, the centers of the four corner pieces
//x1,y1
//x4,y1
//y4,x4
//y4,x1  

gPTPCmd.ptpMode = MOVL_XYZ;
gPTPCmd.x = iniX + x1;
gPTPCmd.y = iniY + y1;
gPTPCmd.z = upZ;
gPTPCmd.r = 0;
SetPTPCmd(&gPTPCmd, true, &gQueuedCmdIndex);
ProtocolProcess();

gPTPCmd.ptpMode = MOVL_XYZ;
gPTPCmd.x = iniX + x4;
gPTPCmd.y = iniY + y1;
gPTPCmd.z = upZ;
gPTPCmd.r = 0;
SetPTPCmd(&gPTPCmd, true, &gQueuedCmdIndex);
ProtocolProcess();
 
gPTPCmd.ptpMode = MOVL_XYZ;
gPTPCmd.x = iniX + x4;
gPTPCmd.y = iniY + y4;
gPTPCmd.z = upZ;
gPTPCmd.r = 0;
SetPTPCmd(&gPTPCmd, true, &gQueuedCmdIndex);
ProtocolProcess();

gPTPCmd.ptpMode = MOVL_XYZ;
gPTPCmd.x = iniX + x1;
gPTPCmd.y = iniY + y4;
gPTPCmd.z = upZ;
gPTPCmd.r = 0;
SetPTPCmd(&gPTPCmd, true, &gQueuedCmdIndex);
ProtocolProcess();

gPTPCmd.ptpMode = MOVL_XYZ;
gPTPCmd.x = iniX + x1;
gPTPCmd.y = iniY + y1;
gPTPCmd.z = upZ;
gPTPCmd.r = 0;
SetPTPCmd(&gPTPCmd, true, &gQueuedCmdIndex);
ProtocolProcess();
  
}

//*****************************************
void wait(){
  do
  {}
  while(1);
}

//*****************************************
void haltMessage(){
Serial2.write(254);
Serial2.write(1);
do
  {
  Serial2.print("dwnZ Error Halt");
  Serial2.write(254);
  Serial2.write(192);
  Serial2.print("Reboot Dobot Ard");
  delay(800);
  Serial2.write(254);
  Serial2.write(1);
  delay(500); 
  }
while(1);
}

//*******************************************************
void adjustStylusUpPosition(){
 
  delay(500);
  //move stylus to pos 11 of puzzle
  //********************************
  gPTPCmd.ptpMode = MOVL_XYZ;
  gPTPCmd.x = iniX;
  gPTPCmd.y = iniY;
  gPTPCmd.z = upZ;   //-19
  gPTPCmd.r = 0;
  SetPTPCmd(&gPTPCmd, true, &gQueuedCmdIndex);
  ProtocolProcess();
  delay(500);   
  
  do
  {
  float sensorValueUp = analogRead(upZPotPin);  //sensorValue = 0 to 1023
  float upValue = sensorValueUp/(1024/upAdjustmentRange);    //max down = 1023/(1024 / 8) = 8  (8*10=80, stored in EEPROM)
  upZ = (-15 + -upValue);   //typ range = -15 to -23 (nom = -19) 
  Serial2.write(254);
  Serial2.write(1);
  delay(20);
  Serial2.println("upZ nom = -19");
  Serial2.write(254);
  Serial2.write(192);
    
  Serial2.print("upZ = ");
  Serial2.print(upZ,1);
  delay(300);

  //finally, when the switch is moved to DONE
  if ((digitalRead(adjustStylusUpZSw)) == LOW)
   {
   delay(200); 
   EEPROM.write(3,-upZ);   //cannot store negative numbers(-19), so must negate first
   delay(100);
   return; 
   }

  }
  while(1);
 }


//*******************************************************
void adjustStylusDownPosition(){

  //NOTE: THIS ROUTINE ALSO DISPLAYS THE POSITION SENSOR READING.
  //A CORRECT dwnZ READING IS ABOUT 425 COUNTS, SO, LET'S SET "EMERGENCY STOP" AT  <350.
  //I.E. IF STYLUS DOESN'T READ AT LEAST 350, WE KNOW IT HAS FALLEN IN
  //A HOLE AND WE NEED TO HALT THE ACTUAL MOVE COMMAND.
  
if ((digitalRead(adjustStylusDownPosSw)) == LOW) 
 {
  delay(500);
  //move stylus to pos1
  //********************************
  gPTPCmd.ptpMode = MOVL_XYZ;
  gPTPCmd.x = iniX;
  gPTPCmd.y = iniY;
  gPTPCmd.z = upZ;   //-19
  gPTPCmd.r = 0;
  SetPTPCmd(&gPTPCmd, true, &gQueuedCmdIndex);
  ProtocolProcess();
  delay(500);   
  
  do
  {
  float sensorValue = analogRead(potInput);  //sensorValue = 0 to 1023
  float downValue = sensorValue/(1024/downAdjustmentRange);    //max down = 1023/(1024 / 8) = 8  (8*10=80, stored in EEPROM)
  dwnZ = (-downValue + -1 + upZ);   //typ range = -16 to -24 (nom = -20) Note: The -1 controls the delta lift value.  upZ = -19.
  Serial2.write(254);
  Serial2.write(1);
  delay(20);
  //Serial2.print("Z adj = -16,-24");  //nominal upZ = -19, Zadj = -16/-24, nom -20
  Serial2.print("dwnZ = ");
  Serial2.print(dwnZ,1);
  Serial2.write(254);   //move to line 2
  Serial2.write(192);
  displacementCount = analogRead(displacementSensorPin);  //425 = dwnZ, set limit at 350, IF <350, STOP!!!!
  Serial2.print("Displace = ");
  Serial2.print(displacementCount);
  delay(300);

  //lower rod by turning pot
  //************************
  //load MOVL,absX,absY,dwnZ(1)
  gPTPCmd.ptpMode = MOVL_XYZ;
  gPTPCmd.x = iniX;
  gPTPCmd.y = iniY;
  gPTPCmd.z = dwnZ;
  gPTPCmd.r = 0;
  SetPTPCmd(&gPTPCmd, true, &gQueuedCmdIndex);
  ProtocolProcess();
  delay(500);         //this delay is important, maybe it needs to longer to allow the Dobot to respond??

  //finally, when the switch is moved to DONE... 
  if ((digitalRead(adjustStylusDownPosSw)) == HIGH)
   {
   delay(200); 
   raiseStylus(); 
   EEPROM.write(2,((downValue + 1) * 10));
   delay(100);
   return; 
   }
  }
  while(1);
 }
}

//**************************************************
void raiseStylus(){
gPTPCmd.ptpMode = MOVL_XYZ;
gPTPCmd.x = iniX;
gPTPCmd.y = iniY;
gPTPCmd.z = upZ;
gPTPCmd.r = 0;
SetPTPCmd(&gPTPCmd, true, &gQueuedCmdIndex);
ProtocolProcess();
}

//***************************************************
void resetNumOfMovesOnLCD(){
  Serial2.write(254);
  Serial2.write(1);
}

//***************************************************
void blinkYel(){
digitalWrite(blinkYelPin, HIGH);
delay(200);
digitalWrite(blinkYelPin, LOW);
delay(200);
}

//***********************************************
void blinkBlu(){
digitalWrite(blinkBluPin, HIGH);
delay(50);
digitalWrite(blinkBluPin, LOW);
delay(50);
}

//***********************************************
void blinkGrn(){
digitalWrite(blinkGrnPin, HIGH);
delay(50);
digitalWrite(blinkGrnPin, LOW);
delay(50);
}

void YelOn()
{digitalWrite(blinkYelPin, HIGH);}

void YelOff()
{digitalWrite(blinkYelPin, LOW);}

void BluOn()
{digitalWrite(blinkBluPin, HIGH);}

void BluOff()
{digitalWrite(blinkBluPin, LOW);}

void GrnOn()
{digitalWrite(blinkGrnPin, HIGH);}

void GrnOff()
{digitalWrite(blinkGrnPin, LOW);}


//*************  END OF PROGRAM ****************
