/* ######################
 *  PUZZLE SOLVER SKETCH 
 * ######################
 * 
 * Project Title: PuzzleBot
 * Purpose: Use a Dobot Magician robot arm to solve a 4x4 sliding puzzle
 * Microcontroller: Arduino Mega 2560
 * 
 * File: PuzzleSolver20r
 * Date: 8/23/23
 * 
 * Written by: Dave Goodsell 
 * dsgoodsell@verizon.net
 * 
 * COM7  COM7  COM7
 * 
 Ver.6: Adding HOME servo, to push KEY button on rear of Dobot.
 Ver.8: HOME servo is working great. There is a chance that this project is done.                
 Ver.9: Not much else to do???? Yeh, right!!!
 Ver.12: Fixing final moves section. I tried moving the 3 into the corner... too difficult to explain and 
         implement with 13 and 8 corners. I like doing "around the corner" moves, easy to explain. 
 Ver.13: More final moves fixing.
 Ver.13a: Screwing around with 14 & 10 rotation so it doesn't move the 14 up, then down (if the 10 and 16 are in
          certain places). 
 Ver.14: This worked for puzzle2 but not for puzzle3. The hell with it.
         HOWEVER, THE NEW IMPROVED FINAL CODE REDUCED PUZZLE2 FROM 94 MOVES, TO 86 MOVES. ZOWIE! 
 Ver.15: Adding Puzzle Select rotary switch display on LCD, as you rotate the switch. Not as good as it seemed. DUMPED!
 Ver.16: Adding "around the corner" sequence that I can call with a button push.
 Ver.17: Also a 14,10 frozen setup to explain how it works. 
 Ver.18: Adding demo of how Dobot physically moves pieces, three moves. num16(blank) MUST be in pos3.
         The "typ mov"(auxButton) triggers it. No display on 32x32.
         The Automatic Shutoff has saved me several times when the stylus dropped into the blank space and was 
         about to forcefully try to move a fixed piece. The Pressure Sensor "sensed" that the stylus was NOT pressing
         on a piece and shut the Dobot off.
         The problem, of course, was caused by ME not initially setting up the sliding puzzle correctly. 
 Ver.18a: Starting code-cleanup for article. 
 Ver.19: More cleanup. 
 Ver.20: Cleanup, including adding the correct bitwise routines for sending parallel bits to other Arduinos. Done!
 Ver.20a,b: Code cleanup 
 Ver.20c: Receiving serial data from camera/Raspi.
 Ver.20d: More serial stuff. It's working!!!! At least the Demo mode. Next is Dobot mode.
 Ver.20e: Dobot mode stuff.
 Ver.20g: Testing Matt's solving sequence YY,YY,YY... YY,ZZ
 Ver.20h: Safety: It's all working, safety copy
 Ver.20i: Continuing...  adding 2 beeps at Go pulse. Raspi is erroniously triggering on Go pulse many times during
          Dobot moving. It may be a ground loop. Quick fix: Use Go button on Raspi panel.BTW & dump Go pulse event in Raspi.
          Success!
 Ver.20j: Trying to clean up display appearance when writing to 32x32 during imageProcessing. It's too jumpy.
 Ver.20k: Changing things. Fall back on 20j if screws up.
 Ver.20m: Look at Line 776, I think it will fix stuff. Yes.... it all works now. Safety copy.
 Ver.20n: Trying auto-triggering now. It false-triggered before, during Dobot moving.
 Ver.20p: Trying to Demo the photo image, no Dobot. And display elapsed time of Dobot solving. 
 Ver.20q: Shortened camera cable by 1/2. Made a difference. Ordered even longer Raspi control cables to allow
          even shorter camera cable if desired.         
*/

#include <Servo.h>
Servo homeServo;

//*************************************************************************************
//       List of correctly shuffled matrices, plus illegal shuffle in Matrix4
//*************************************************************************************
//Rotary Switch selected Matrices:
//********************************
//Switch position 0      = {imageMatrix received from Raspberry Pi/Camera Image Processor}; 
//Switch positions 1-5, see below: 
byte startingMatrix1[17] = {0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16};    //solved configuration
byte startingMatrix2[17] = {0,5,3,16,6,8,7,4,15,14,12,10,13,1,9,11,2};    //typical matrix, 86 moves
byte startingMatrix3[17] = {0,1,2,3,6,5,8,16,4,7,14,13,15,9,12,10,11};    //demo of 1,2,3,4 corner technique
byte startingMatrix4[17] = {0,1,2,3,4,5,6,7,8,9,10,11,12,13,15,14,16};    //config with 14,15 switched (NOT solvable)
byte startingMatrix5[17] = {0,1,2,3,4,5,6,7,8,9,14,10,12,13,16,11,15};    //demo of 14,10 alignment for final moves

//More unused shuffled matrices:
//byte startingMatrixX[17] = {0,1,2,3,4,5,6,7,8,9,11,10,12,13,15,14,16};  //config with 14,15 & 10,11 switched (solvable)
//byte startingMatrixX[17] = {0,8,12,2,7,15,1,5,9,13,6,10,3,11,4,16,14};  //135 moves
//byte startingMatrixX[17] = {0,1,4,9,7,14,16,13,5,8,6,11,2,10,3,12,15};  //126 moves
//byte startingMatrixX[17] = {0,5,14,7,11,8,12,6,15,10,16,1,13,9,4,3,2};  //127 moves
//byte startingMatrixX[17] = {0,8,13,6,2,11,10,14,1,9,12,7,16,3,15,4,5};  //147 moves
//byte startingMatrixX[17] = {0,13,7,9,2,14,3,11,4,1,10,12,8,6,5,15,16};  //114 moves
//byte startingMatrixX[17] = {0,13,14,12,6,16,7,8,1,11,3,2,10,5,4,15,9};  //139 moves
 
//***************************************************************************************
//                            ARRAYS TO SOLVE EACH NUMBER
//***************************************************************************************
byte Num1Array[256] = {
  0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,\
  2,0,7,8,1,5,6,7,5,9,10,11,9,13,14,15,\
  2,3,0,8,1,5,6,7,5,9,10,11,9,13,14,15,\
  2,3,4,0,1,5,6,7,5,9,10,11,9,13,14,15,\
  
  5,1,2,3,0,2,6,7,10,6,10,11,9,10,11,12,\
  2,6,2,3,6,2,3,7,5,11,7,8,9,10,11,12,\
  2,3,7,3,6,7,0,4,5,6,12,8,9,10,11,12,\
  2,3,7,8,6,7,8,0,5,6,7,11,9,10,11,12,\

  5,1,2,3,9,5,6,7,0,6,7,8,14,10,11,12,\
  5,6,2,3,9,10,6,7,10,0,7,8,9,13,14,15,\
  5,6,7,3,9,10,11,7,10,11,0,8,9,10,14,15,\
  5,6,7,8,9,10,11,12,10,11,12,0,9,10,11,15,\

  5,1,2,3,9,5,6,7,13,9,10,11,0,10,14,15,\
  2,6,2,3,6,10,6,7,10,14,10,11,14,0,11,15,\
  2,6,2,3,6,10,6,7,13,14,15,11,14,15,0,12,\
  2,6,2,3,6,10,6,7,13,14,15,11,14,15,16,0
};

byte Num2Array[256] = {
  0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,\
  0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,\
  0,3,0,8,6,2,6,7,5,6,7,8,9,10,11,12,\
  0,3,4,0,6,2,3,7,5,6,7,8,9,10,11,12,\

  0,6,7,3,0,5,6,7,10,6,7,8,9,10,11,12,\
  0,6,2,3,9,0,3,4,10,11,7,8,9,10,11,12,\
  0,3,7,3,6,2,0,4,5,6,12,8,9,10,11,12,\
  0,3,4,8,6,7,8,0,5,6,7,11,9,10,11,12,\

  0,6,2,8,9,5,6,7,0,6,7,8,14,10,11,12,\
  0,6,2,3,6,10,6,7,5,0,7,8,9,15,11,12,\
  0,6,7,3,2,7,11,7,5,6,0,8,9,10,16,12,\
  0,6,7,8,9,10,11,12,10,11,12,0,9,10,11,15,\

  0,6,7,8,9,10,11,12,13,9,10,11,0,10,11,12,\
  0,6,7,8,9,10,11,12,10,14,10,11,9,0,11,12,\
  0,3,7,3,6,7,11,7,10,11,15,11,9,10,0,12,\
  0,3,4,8,6,7,8,12,10,11,12,16,9,10,11,0
};

byte Num3Array[256] = {
  0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,\
  0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,\
  0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,\
  0,0,4,0,6,7,3,7,5,6,7,8,9,10,11,12,\

  0,0,7,8,0,5,6,7,10,6,7,8,9,10,11,12,\
  0,0,7,3,9,0,6,7,10,11,7,8,9,10,11,12,\
  0,0,7,3,9,10,0,4,10,11,12,8,9,10,11,12,\
  0,0,4,8,6,7,3,0,5,6,7,11,9,10,11,12,\

  0,0,7,8,9,5,6,7,0,6,7,8,14,10,11,12,\
  0,0,7,8,6,10,6,7,5,0,7,11,9,15,11,12,\
  0,0,7,3,6,7,11,7,5,6,0,8,9,10,16,12,\
  0,0,4,8,6,7,8,12,5,6,7,0,9,10,11,15,\

  0,0,7,8,9,10,11,12,13,9,10,11,0,10,11,12,\
  0,0,7,8,6,10,11,12,10,14,10,11,9,0,11,12,\
  0,0,7,8,6,7,11,7,10,11,15,11,9,10,0,12,\
  0,0,4,8,6,7,8,12,10,11,12,16,9,10,11,0
};

byte Num4Array[256] = {
  0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,\
  0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,\
  0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,\
  0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,\

  0,0,0,8,0,5,6,7,10,6,7,8,9,10,11,12,\
  0,0,0,8,9,0,6,7,10,11,7,8,9,10,11,12,\
  0,0,0,8,9,10,0,7,10,11,12,8,9,10,11,12,\
  0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,\

  0,0,0,8,9,5,6,7,0,6,7,8,14,10,11,12,\
  0,0,0,8,6,10,6,7,5,0,7,8,9,15,11,12,\
  0,0,0,8,6,7,11,7,5,6,0,8,9,10,16,12,\
  0,0,0,8,6,7,8,12,5,6,7,0,9,10,11,15,\

  0,0,0,8,9,10,11,12,13,9,10,11,0,10,11,12,\
  0,0,0,8,9,10,6,7,13,14,10,11,14,0,11,12,\
  0,0,0,8,6,7,11,7,10,11,15,11,14,15,0,12,\
  0,0,0,8,6,7,8,12,10,11,12,16,9,10,11,0
};

byte Num5Array[256] = {
  0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,\
  0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,\
  0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,\
  0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,\

  0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,\
  0,0,0,0,6,0,11,7,5,9,10,11,9,10,11,15,\
  0,0,0,0,6,7,0,12,5,6,10,11,9,10,11,15,\
  0,0,0,0,6,7,8,0,5,6,7,11,9,10,11,15,\

  0,0,0,0,9,5,6,7,0,6,7,8,14,10,14,15,\
  0,0,0,0,9,10,6,7,10,0,7,8,9,13,14,15,\
  0,0,0,0,9,10,11,7,10,11,0,8,9,10,14,14,\
  0,0,0,0,9,10,11,12,10,11,12,0,9,10,11,15,\

  0,0,0,0,9,5,6,7,13,9,10,11,0,10,11,12,\
  0,0,0,0,6,10,6,7,10,14,10,11,14,0,11,15,\
  0,0,0,0,9,10,11,7,13,14,10,11,14,15,0,12,\
  0,0,0,0,9,10,11,12,10,11,15,16,14,15,16,0  
};

byte Num6Array[256] = {
  0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,\
  0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,\
  0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,\
  0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,\

  0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,\
  0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,\
  0,0,0,0,0,7,0,12,0,6,10,11,0,10,11,12,\
  0,0,0,0,0,7,8,0,0,6,7,11,0,10,11,12,\

  0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,\
  0,0,0,0,0,10,6,7,0,0,7,8,0,15,11,12,\
  0,0,0,0,0,7,11,7,0,11,0,8,0,10,14,15,\
  0,0,0,0,0,10,8,12,0,11,12,0,0,10,11,15,\

  0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,\
  0,0,0,0,0,10,6,7,0,14,10,11,0,0,11,12,\
  0,0,0,0,0,10,6,7,0,14,15,11,0,15,0,12,\
  0,0,0,0,0,10,11,15,0,14,15,11,0,15,16,0  
};

byte Num7Array[256] = {
  0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,\
  0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,\
  0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,\
  0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,\

  0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,\
  0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,\
  0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,\
  0,0,0,0,0,0,8,0,0,7,11,7,0,10,11,12,\

  0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,\
  0,0,0,0,0,0,11,12,0,0,10,11,0,15,16,12,\
  0,0,0,0,0,0,11,7,0,14,0,8,0,15,16,12,\
  0,0,0,0,0,0,11,12,0,11,12,0,0,10,11,15,\

  0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,\
  0,0,0,0,0,0,11,7,0,14,10,11,0,0,11,12,\
  0,0,0,0,0,0,11,7,0,11,15,11,0,10,0,12,\
  0,0,0,0,0,0,11,12,0,11,12,16,0,15,16,0  
};

//checked again on 3/26
byte Num8Array[256] = {
  0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,\
  0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,\
  0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,\
  0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,\

  0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,\
  0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,\
  0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,\
  0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,\
  
  0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,\
  0,0,0,0,0,0,0,12,0,0,10,11,0,15,11,12,\
  0,0,0,0,0,0,0,12,0,14,0,11,0,15,16,12,\
  0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,\

  0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,\
  0,0,0,0,0,0,0,12,0,14,10,11,0,0,11,12,\
  0,0,0,0,0,0,0,12,0,11,15,11,0,10,0,12,\
  0,0,0,0,0,0,0,12,0,11,12,16,0,15,11,0
};
 
byte Num8toPos8WhenNum8inPos12andNum16inPos10Array[8][2]{    //num16 in pos11, using "around-the-corner" moves
  6,10,\
  7,6,\
  8,7,\
  12,8,\
  11,12,
  7,11,\
  6,7,\
  10,6
};


byte Num9Array[256] = {
  0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,\
  0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,\
  0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,\
  0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,\

  0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,\
  0,0,0,0,0,0,11,12,10,6,10,11,9,10,11,15,\
  0,0,0,0,0,7,0,12,10,6,10,11,9,10,11,12,\
  0,0,0,0,0,7,8,0,10,6,7,11,9,10,11,12,\

  0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,\
  0,0,0,0,0,7,11,12,10,0,15,16,9,13,14,15,\
  0,0,0,0,0,10,6,7,10,11,0,16,9,10,14,15,\
  0,0,0,0,0,10,11,7,10,11,12,0,9,10,11,15,\

  0,0,0,0,0,10,11,12,13,9,10,11,0,10,11,12,\
  0,0,0,0,0,10,11,13,7,14,10,11,14,0,11,12,\
  0,0,0,0,0,10,11,7,10,11,15,11,14,15,0,12,\
  0,0,0,0,0,10,8,12,10,11,12,16,14,15,16,0
};

byte Num13Array[256] = {
  0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,\
  0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,\
  0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,\
  0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,\

  0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,\
  0,0,0,0,0,0,11,12,0,6,10,11,14,10,14,15,\
  0,0,0,0,0,7,0,12,0,6,10,11,14,10,14,15,\
  0,0,0,0,0,7,8,0,0,6,7,11,14,10,14,15,\

  0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,\
  0,0,0,0,0,7,11,12,0,0,15,16,14,10,14,15,\
  0,0,0,0,0,10,6,7,0,11,0,16,14,10,14,15,\
  0,0,0,0,0,10,11,7,0,11,12,0,14,10,11,15,\

  0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,\
  0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,\
  0,0,0,0,0,10,6,7,0,14,10,11,14,15,0,12,\
  0,0,0,0,0,10,11,7,0,14,15,11,14,15,16,0
};


byte Num13toPos13WhenNum13inPos14andNum16inPos10Array[9][2]{    //num16 in pos10, using "around-the-corner" moves
  6,10,\
  5,6,\
  9,5,\
  13,9,\
  14,13,\
  10,14,\
  9,10,\
  5,9,\
  6,5
};

byte Num13toPos13WhenNum13inPos14andNum16inPos15Array[2][2]{    //num16 in pos15, using "around-the-corner" moves
  11,15,\
  10,11  
};

//Final Moves To Solve 10,11,12,14,15
//************************************
//Strategy: Move num14 to pos10, num10 to pos11, rotate CCW, do final moves

byte Num14toPos10Array[256] = {
  0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,\
  0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,\
  0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,\
  0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,\

  0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,\
  0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,\
  0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,\
  0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,\

  0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,\
  0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,\
  0,0,0,0,0,0,0,0,0,11,0,16,0,10,14,15,\
  0,0,0,0,0,0,0,0,0,11,12,0,0,10,11,15,\

  0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,\
  0,0,0,0,0,0,0,0,0,14,10,11,0,0,11,15,\
  0,0,0,0,0,0,0,0,0,14,15,11,0,15,0,12,\
  0,0,0,0,0,0,0,0,0,11,12,16,0,10,16,0
};

//*****************************************************

//matrix coordinates of the upper-left corners of each square in the LED display panel, squares 1 to 16
//***************************************************************
byte coordinatesInLEDMatrix[17][2] = {
  0,0,\
  1,1,\
  8,1,\
  15,1,\
  22,1,\
  1,8,\
  8,8,\
  15,8,\
  22,8,\
  1,15,\
  8,15,\
  15,15,\
  22,15,\
  1,22,\
  8,22,\
  15,22,\
  22,22
};

byte brighteningNumberPinArray[4] = {2,3,4,5};
byte fromPositionPinArray[4] = {A0,A1,A2,A3};
byte toPositionPinArray[4] = {A4,A5,A6,A7};
byte puzzleNumberLEDPinArray[3] = {29,22,33};
byte Matrix[17];
byte startingMatrix[17];  
byte receivingRX1Buffer[100];
byte imageMatrix[16];

//**********************************************************************
//           GLOBAL VARIABLES
//**********************************************************************
byte From1;
byte To1;   
byte From;
byte To;
byte num;
byte posToMoveToPosOf16; 
byte Row;
byte Pos;
byte test;
byte test2;
int  pass;
int  Totalpass;
byte testingStartPosOfNum1;
byte x;
char c;
byte idx;
byte sidx;
int  moves;  
int  sendMovesToDobotButton = 53;
int  sendArrayButton = 51;
int  bigYellowButtonPin = 50;
int  bigYellowLEDPin = 48;

//Servo is pin9, declared in Setup

int LEDGrn = 31;
int LED = 23;
int LEDYel = 29;
int LEDBlu = 35;

int beeper = 39;
int testPin = 25;
int singleStepButtonPin = 49;         //also HOME button

int CTSFromDobot = A8;
int DataReadyToDobot = A9;
int ResetToDobot = A10;

int dobotMakingMoveRightNow = 27;     //handshake pin for display sync
int RequestToSendSolveMoves = 41;     
int RequestToSendStartingArray = 43;
int DisplayReadyToReceiveData = 45;
int resetPinDisplayArduino = 47;     //connected directly to Display Arduino reset pin

int solveSpeed;
int defaultSolveSpeed = 200;  //this the speed between numbers, it is changed for Matrix1,3,5
int solveSpeedActual = 200;
int Matrix3PinDelay = 1500;
int Matrix5PinDelay = 1000;
int fastSolveDelay = 40;  //100
int normalSolveDelay = 2000;

int brightOnePin = 2;
int brightTwoPin = 3;
int brightFourPin = 4;
int brightEightPin = 5;

int goCmdToRaspiPin = 6;

int oneLED = 29;
int twoLED = 22;
int fourLED = 33;
int auxButtonPin = 37;
int servoRelayPowerPin = 24;
int blankEaselPin = 26;

unsigned long startTime;
unsigned long dobotStartTime;
int iteration;
int iteration2;

bool puzzleSolvedFlag;
bool calculateNextMoveFlag;   //not used????
bool imageMatrixReceivedAndLoadedFlag;

bool programMode = 0;   //Mode0 = testSolvingAlgorithm using Display; Mode1 = all-up, Display follows Dobot Moves of puzzle pieces
bool fastSolve = 0;
bool sendDigitFlag;
bool interimImageFlag;

int  Digit;

int Matrix0Pin = 52;  
int Matrix1Pin = A15;
int Matrix2Pin = A14;
int Matrix3Pin = A13;
int Matrix4Pin = A12;
int Matrix5Pin = A11;

byte posOfNum1;
byte posOfNum2;
byte posOfNum3;
byte posOfNum4;
byte posOfNum5;
byte posOfNum6;
byte posOfNum7;
byte posOfNum8;
byte posOfNum9;
byte posOfNum10;
byte posOfNum11;
byte posOfNum12;
byte posOfNum13;
byte posOfNum14;
byte posOfNum15;
byte posOfNum16;

//************************************************************************************* 
//*************************************************************************************
void setup(){
  
Serial.begin (115200); //Arduino program monitor
Serial1.begin(9600);   //RX1 input from Raspi TX, TX1 output to LCD Display RX
Serial2.begin(9600);
Serial3.begin(9600);

homeServo.attach(9);

pinMode(sendMovesToDobotButton, INPUT_PULLUP);
pinMode(sendArrayButton, INPUT_PULLUP);

pinMode(LED, OUTPUT);  //red LED
pinMode(LEDGrn, OUTPUT);
pinMode(LEDYel, OUTPUT);
pinMode(LEDBlu, OUTPUT);

pinMode(beeper, OUTPUT);
pinMode(testPin, OUTPUT);
pinMode(singleStepButtonPin, INPUT_PULLUP);
pinMode(bigYellowButtonPin, INPUT_PULLUP);
pinMode(bigYellowLEDPin, OUTPUT);
pinMode(goCmdToRaspiPin, OUTPUT);

pinMode(RequestToSendSolveMoves, OUTPUT);       //pin41    The moves start after XMTR receives DTR from RCVR
pinMode(RequestToSendStartingArray, OUTPUT);    //pin43    The array gets sent after a DTR is received
pinMode(DisplayReadyToReceiveData, INPUT);      //pin45
//pinMode(RequestToSendMovesToDobot, OUTPUT);   //pin47    Not used

pinMode(A0, OUTPUT); 
pinMode(A1, OUTPUT);
pinMode(A2, OUTPUT);
pinMode(A3, OUTPUT);
pinMode(A4, OUTPUT); 
pinMode(A5, OUTPUT);
pinMode(A6, OUTPUT);
pinMode(A7, OUTPUT); 

pinMode(dobotMakingMoveRightNow, INPUT);   //pin 27    
pinMode(CTSFromDobot, INPUT);       //A8
pinMode(DataReadyToDobot, OUTPUT);  //A9
pinMode(ResetToDobot, OUTPUT);      //A10
pinMode(resetPinDisplayArduino, OUTPUT);    //connected directly to Display Arduino reset pin
pinMode(servoRelayPowerPin, OUTPUT);        //Eliminates harmful servo glitch during bootup
pinMode(blankEaselPin, OUTPUT); 

pinMode(brightOnePin, OUTPUT);   //signal to display to make number brighter as the number FIRST starts getting solved, like Solve num1.
pinMode(brightTwoPin, OUTPUT);
pinMode(brightFourPin, OUTPUT);
pinMode(brightEightPin, OUTPUT); 

pinMode(Matrix0Pin, INPUT_PULLUP);
pinMode(Matrix1Pin, INPUT_PULLUP); 
pinMode(Matrix2Pin, INPUT_PULLUP);
pinMode(Matrix3Pin, INPUT_PULLUP); 
pinMode(Matrix4Pin, INPUT_PULLUP);
pinMode(Matrix5Pin, INPUT_PULLUP);

pinMode(oneLED, OUTPUT);
pinMode(twoLED, OUTPUT);
pinMode(fourLED, OUTPUT);
pinMode(auxButtonPin, INPUT_PULLUP);

digitalWrite(brightOnePin, LOW);
digitalWrite(brightTwoPin, LOW);
digitalWrite(brightFourPin, LOW);
digitalWrite(brightEightPin, LOW);

homeServo.write(150);   //moves away from button
delay(15);
digitalWrite(servoRelayPowerPin, HIGH);  //servo on
delay(500);
digitalWrite(servoRelayPowerPin, LOW);   //servo off

digitalWrite(resetPinDisplayArduino, HIGH);
//digitalWrite(goCmdToRaspiPin , LOW);

//Initialize 2-line LCD
Serial1.write(254);
Serial1.write(1);
delay(15);
Serial1.print("Ready");
Serial1.write(254);
Serial1.write(192);
Serial1.print("Press 1st & 2nd");
delay(15);

//Load startingMatrix in depending on rotary switch setting
if ((digitalRead(Matrix1Pin)) == LOW){for (idx=0;idx<=16;idx++)\
   {startingMatrix[idx] = startingMatrix1[idx];}setPuzzleNumLEDs(1);}
if ((digitalRead(Matrix2Pin)) == LOW){for (idx=0;idx<=16;idx++)\
   {startingMatrix[idx] = startingMatrix2[idx];}setPuzzleNumLEDs(2);}
if ((digitalRead(Matrix3Pin)) == LOW){for (idx=0;idx<=16;idx++)\
   {startingMatrix[idx] = startingMatrix3[idx];}setPuzzleNumLEDs(3);} 
if ((digitalRead(Matrix4Pin)) == LOW){for (idx=0;idx<=16;idx++)\
   {startingMatrix[idx] = startingMatrix4[idx];}setPuzzleNumLEDs(4);}
if ((digitalRead(Matrix5Pin)) == LOW){for (idx=0;idx<=16;idx++)\
   {startingMatrix[idx] = startingMatrix5[idx];}setPuzzleNumLEDs(5);}

beep();
beep();
beep();


}  //end setup

//*******************************************************************************************************************************************
//******************************************************************************************************************************************* 
//*******************************************************************************************************************************************
//******************************************************************************************************************************************* 
//                                              START OF PROGRAM
//*******************************************************************************************************************************************
//******************************************************************************************************************************************* 
//*******************************************************************************************************************************************
//*******************************************************************************************************************************************  
void loop(){

//COM7  COM7  COM7  COM7 

//***********************************************************************************
//          CONTINUOUSLY CHECKING FOR BUTTON PUSHES AND OTHER CONDITIONS
//***********************************************************************************

//************************************************************************
//Dobot commanded to HOME
//CAUTION: THE SERVO MUST PRESS THE HOME BUTTON FOR AT LEAST 2 SECS OR THE ARM WILL 
//MAKE RAPID MECHANICALLY HARMFUL MOVES.
if ((digitalRead(singleStepButtonPin)) == LOW)  //also HOME button (dual purpose)
  {
   beep();
   digitalWrite(servoRelayPowerPin, HIGH);  //servo on
   delay(500); 
   homeServo.write(123);   //presses button
   delay(3000);            //CAUTION: THIS PULSE MUST BE AT LEAST 2 SECONDS
   homeServo.write(150);   //141 normally 
   delay(500);
   digitalWrite(servoRelayPowerPin, LOW);  //servo off
  }

//************************************************************************
//Dobot demonstrates one move
if ((digitalRead(auxButtonPin)) == LOW)
   {
   beep(); 
   Move(2,3);
   Move(1,2);
   Move(5,1);
   Move(17,17);
   delay(1000); 
   }
   
//****************************************************************************************   
//****************************************************************************************
//DOBOT Button 53(DO): Dobot physically solves imaged puzzle using camera (Mode 0 only)
//This button starts Matt's image solving routine & Dobot moves
//****************************************************************************************
//****************************************************************************************

if ((digitalRead(sendMovesToDobotButton)) == LOW)  //pin 53 DOBOT button
   {
   beep();
    
   blink();  //red
   fastSolve = 0;
   programMode = 1;        //all-up operation, with Dobot moving puzzle pieces and Display sync'd to moving pieces
   puzzleSolvedFlag = 0;   //this is true start of the whole program(s)
   //digitalWrite(blankEaselPin, LOW);
   rstDisplayArduino();
      
   //********************************************
     //Matt: Grn READY LED is already on, actually since initial bootup
     //Dave: sends Go signal to Raspi, then waits for Serial Data to come in
     //Matt: photo light comes on
     //Matt: image matching: 1:14 min, BEEPS each time a match is made
     //Dave: Displays latest match on 32x32 display
     //Matt: If Red ERROR LED comes on:
     //Dave: do 1st & 2nd reboot, small button 53 "DO" DOBOT(4 beeps), Raspi RESET button 
     //Matt: sends serial data to Arduino
     //Dave: if data is YY, sends to 32x32, but NO solving
     //Dave: if data is ZZ, send to 32x32, DO solving
     //Matt: Yel BIG button LED on at finish(ZZ)
     //Dave: Eyeball confirm match, then hit yel button  
   //*******************************************

   //****************************************************************
   //send Go pulse to Raspi, to start Matt's image processing program  
    beep();
    beep();
    digitalWrite(goCmdToRaspiPin, HIGH);
    delay(1000);
    digitalWrite(goCmdToRaspiPin, LOW);
   //**************************************************************** 

   //elapsed time display
   iteration = 1;
   iteration2 = 0;
   startTime = millis();

   //IMPORTANT LOOP TO DISPLAY INTERIM IMAGE-SOLVING SEQUENCE YY IF interimImageFlag is HIGH
   //*****************************************************************************************
   do    //do loop to display Matt's image solving sequence, then final startingMatrix
   {
    loadImageMatrixFromRaspi();   //do loop waiting for serial data
    sendArrayToDisplay();   
    iteration2++;
    delay(1000);
    beep();
   }
   while(iteration2 != 15);   //loop stops when Matt has completely determined configuration
 
   beep();
   beep(); 
       
   do     //wait for Big Yellow to start the Dobot moving
   {
   digitalWrite(bigYellowLEDPin, HIGH);
   int("Then Hit Yellow");
   
   delay(500);
   digitalWrite(bigYellowLEDPin, LOW); 
   }
   while((digitalRead(bigYellowButtonPin)) == HIGH);  //used to be Dobot button 

   digitalWrite(bigYellowLEDPin, LOW);
   
   Serial1.write(254);
   Serial1.write(1);  //clear LCD
   delay(50);
   dobotStartTime = millis();
      

   //#######################################    WHY DIDN'T I ADD THIS EARLIER???LORDY
   digitalWrite(RequestToSendSolveMoves, HIGH); 
   waitForDisplayReadyToReceiveData();
   digitalWrite(RequestToSendSolveMoves, LOW);
   //###################################### 
    
   doSolving();   //this solves until puzzleSolvedFlag = 1
   //this is a do loop that waits for CTS from the Dobot, latches data to pins, then lets program calculate next Move
   //by releasing "sendMoveToDisplay()"  (note: sendMove.. includes "doPositionMove") 
   //checkIfDobotIsReadyThenSendMove(); //moved to sendMoveToMatrix()

   } 

//*************************************************************************************************
//*************************************************************************************************
//DEMO Button 51(DM): LED Matrix solves demo puzzles, including imageMatrix  (but NO Dobot action)
//*************************************************************************************************
//*************************************************************************************************

if ((digitalRead(sendArrayButton)) == LOW)    //pin 51 DM button    
  {
  beep();
  beep();
 
  if ((digitalRead(Matrix0Pin)) == LOW)
    {
    solveImageDemo();   //same as pushing DO button, this function is at end of the program
    loop();  
    }
  
  Serial1.write(254);
  Serial1.write(1);
  rstDisplayArduino();
  fastSolve = 1;    //exercise solving algorithm using selected Matrices, with Display confirming Moves 
  blink();
  digitalWrite(RequestToSendStartingArray, HIGH);
  waitForDisplayReadyToReceiveData();
  digitalWrite(RequestToSendStartingArray, LOW); 
  transmitStartingMatrix();
  
  Serial1.write(254);
  Serial1.write(1);
  
  //if ((digitalRead(Matrix0Pin)) == LOW)
  //{
  //defaultSolveSpeed = 1000;
  //solveImageDemo();   //same as pushing DO button
  //loop();  
  //}
      
  if ((digitalRead(Matrix1Pin)) == LOW)
    {
    Serial1.print("1.'Solved' Demo");
    Serial1.write(254);
    Serial1.write(192);
    Serial1.print("Press Yel button");
    defaultSolveSpeed = 1000;   
    }
  
  if ((digitalRead(Matrix2Pin)) == LOW)
    {
    Serial1.print("2. Typ Scrambled");
    Serial1.write(254);
    Serial1.write(192);
    Serial1.print("  (86 moves)");
    }
  
  if ((digitalRead(Matrix3Pin)) == LOW)
    {
    Serial1.print("3. Demo 1,2,3,4");
    Serial1.write(254);
    Serial1.write(192);
    Serial1.print("corner solving");
    }
    
  if ((digitalRead(Matrix4Pin)) == LOW)
    {
    Serial1.print("4. 14&15 Switch");
    Serial1.write(254);
    Serial1.write(192);
    Serial1.print("   NOT Solvable");
    brightenNum14();
    brightenNum15();
    }
    
  if ((digitalRead(Matrix5Pin)) == LOW)
    {
    Serial1.print("5. Positioning");
    Serial1.write(254);
    Serial1.write(192);
    Serial1.print("14&10 for final");
    //brightenNum10();  //this causes big problems, do not use it here
    brightenNum14();
    }

  //Big Yellow Button
  digitalWrite(bigYellowLEDPin, HIGH);  
  do     //wait for another button push to start the actual solving
  {}
  while((digitalRead(bigYellowButtonPin)) == HIGH);  //used to be Demo button 
  beep();
  digitalWrite(bigYellowLEDPin, LOW);
  
  digitalWrite(RequestToSendSolveMoves, HIGH); 
  waitForDisplayReadyToReceiveData();
  digitalWrite(RequestToSendSolveMoves, LOW); 
  
  doSolving();
  }
  
} //end of main loop()

//*********************************************************************************************************************************
//*********************************************************************************************************************************
//*********************************************************************************************************************************
//                              FUNCTIONS AND ROUTINES, MAJOR AND MINOR
//*********************************************************************************************************************************
//*********************************************************************************************************************************
//*********************************************************************************************************************************

void loadImageMatrixFromRaspi(){  //process data coming from Raspi and camera, then set the flag

//Processes two types of data from Matt
//1. YY data, that shows the interim results as the image is solved (typ 15 steps)
//2. ZZ data, the final solution
//3. The next step is to have the Dobot actually move the pieces

if (iteration == 1)
{
Serial1.write(254);
Serial1.write(1);
Serial1.print("Taking Photo of");
Serial1.write(254);
Serial1.write(192);
Serial1.print("Puzzle to Solve");
}
else
{
unsigned long currentTime = millis();
unsigned long elapsedTime = currentTime - startTime;
elapsedTime = int(elapsedTime/1000);

Serial1.write(254);
Serial1.write(1);
Serial1.print("Matching: #");
Serial1.print(iteration);
Serial1.write(254);
Serial1.write(192);
Serial1.print("Elapsed: ");
Serial1.print(elapsedTime);
Serial1.print("secs");
}

iteration++;
interimImageFlag = LOW;

idx = 0;
do                      //wait for, then process for incoming serial data
{
if (Serial1.available())
 {
  byte c = Serial1.read();
  if (c == 89)(interimImageFlag = HIGH);  //YY
  if (c == 90)(interimImageFlag = LOW);   //ZZ
  Serial.print(c);
  if ((c >= 48) && (c <= 57))
    {
    receivingRX1Buffer[idx] = c;   //load digits in my Buffer
    idx++;
    }
 }
} 
while (idx <=31); 
 
//while(Serial.read() >= 0); //flush THE buffer, not used at moment

//convert to 16 numbers decimal Matrix
//*************************************
for (idx=0;idx<=31;idx+=2)
 {
  x= receivingRX1Buffer[idx];
  if (x == 48)
  {
  imageMatrix[int(idx/2)] = (receivingRX1Buffer[idx+1] - 48);  
  }
  else
  {
  imageMatrix[int(idx/2)] = (((receivingRX1Buffer[idx] - 48)*10) + (receivingRX1Buffer[idx+1] - 48)); 
  }
 }

//Loading startingMatrix
//**************************
for (idx=0;idx<=15;idx++)
 {
  startingMatrix[0] = 0;                      //the startingMatrix always starts with a 0, then 16 pos(1-16)
  startingMatrix[idx+1] = imageMatrix[idx];   //THIS IS THE BIG STEP FOR MODE 0
  //Serial.print(imageMatrix[idx]);
  //Serial.print(" ");
 }

imageMatrixReceivedAndLoadedFlag = HIGH;

//beep();
//beep();
//beep();
}  

//**************************************************************
void rstDisplayArduino(){
  digitalWrite(resetPinDisplayArduino, LOW);   //pulls the actual Reset pin low for a bit
  delay(500);
  digitalWrite(resetPinDisplayArduino, HIGH);
  delay(1000);
}

//*******************************************************
void sendArrayToDisplay(){       
  //blink();
  delay(200);   //2000  ?????
  digitalWrite(RequestToSendStartingArray, HIGH);
  //Serial.println("Request To Send");
  waitForDisplayReadyToReceiveData();
  //Serial.println("Display Ready");
  digitalWrite(RequestToSendStartingArray, LOW); 
  //Serial.println("Sending Array");
  transmitStartingMatrix();               //also sends YY and ZZ so 32x32 display can "blank" between interim images (flagged)
  //blink();
  delay(500);
  digitalWrite(RequestToSendSolveMoves, HIGH); 
  waitForDisplayReadyToReceiveData();
  digitalWrite(RequestToSendSolveMoves, LOW); 
}

//******************************************************************************
void checkIfDobotIsReadyForNextMove(){
  
  //Actually this handshake does NOT sync the Dobot. The Dobot has a buffer that stores all the Moves
  //then does them at its own pace. I need to do yet another handshake line(s), tied to the Dobot Arduino
  //"ProtocolProcess()". 
  // I JUST PUT IN 2.5 SEC DELAY AND LET IT GO. A 3 SEC DELAY WAS TOO MUCH FOR THE DOBOT.
  //********************************************************
  //wait for CTS comes from Dobot Control Arduino
  do
  {}
  while((digitalRead(CTSFromDobot)) == LOW);
  BluOn();
  
  //when CTS goes high 
  latchMoveToPins(From,To); 
  
  //after data latched to A0-A7 pins
  digitalWrite(DataReadyToDobot, HIGH);
  YelOn();
    
  //wait for Dobot to send back a CTS LOW
  do
  {}
  while((digitalRead(CTSFromDobot)) == HIGH);
  BluOff();
  //when CTS goes LOW

  //close data ready window
  digitalWrite(DataReadyToDobot, LOW);
  YelOff();

  do       //pause if button depressed, allows single stepping
  {}
  while((digitalRead(singleStepButtonPin)) == LOW);
     
  if (fastSolve == 1)
     {
     delay(fastSolveDelay);
     }
  else
     {
     delay(normalSolveDelay);  // norm: 2000ms give the Dobot a chance to make the actual Move
     lcdDisplayElapsedDobotTime();
     }
  //THIS IS THE BIG DELAY POINT!!!! 
   
}

//**********************************************************
void latchMoveToPins(int from, int to){    //ex: 3,7
GrnOn(); 

for (int idx=0;idx<=3;idx++)    
  {
  int a = (1 << idx);   //bitwise shift, 1,2,4
  int b = (from & a);   //bitwise AND (from AND 1,2,4)
  if (b == a)           //if bit == true, write to pin
    {
    digitalWrite(fromPositionPinArray[idx], HIGH);  //A0,A1,A2,A3 
    }
    else
    {
    digitalWrite(fromPositionPinArray[idx], LOW);
    }
  }

for (int idx=0;idx<=3;idx++)  
  {
  int a = (1 << idx);  //bitwise shift, 1,2,4
  int b = (to & a);    //bitwise AND (to AND 1,2,4)
  if (b == a)          //if bit == true, write to pin
    {
    digitalWrite(toPositionPinArray[idx], HIGH);   //A4,A5,A6,A7
    }
    else
    {
    digitalWrite(toPositionPinArray[idx], LOW);
    }
  }

delay(200); 
GrnOff();
}

//*********************************************************
void doSolving(){

//if ((digitalRead(Matrix0Pin))== LOW)  //can't use this for DO(dobot) solving, needs to be 2.5sec???
 //{
 //fastSolve = 1;  
 //solveSpeed = 200; //not applicable I guess   
 //}
if ((digitalRead(Matrix1Pin))== LOW)
  {
  solveSpeed = 1000;
  }
else if ((digitalRead(Matrix3Pin))== LOW)
  {
  solveSpeed = Matrix3PinDelay;
  }
else
  {
  solveSpeed = defaultSolveSpeed;
  }

loadStartingMatrixInMatrix(); 
display3DMatrix();
//beep;
//beep;
//beep;
//beep;
moves = 0;

//1111111111111111111111111111111
solveNum1();   //1
//lcdDisplayElapsedDobotTime();
delay(solveSpeed);  //between number pause

//2222222222222222222222222222222
solveNum2();   //2
delay(solveSpeed);

//3333333333333333333333333333333
solveNum3();   //3
delay(solveSpeed);
 
//4444444444444444444444444444444
solveNum4();   //4
delay(solveSpeed);

if ((digitalRead(Matrix3Pin)) == LOW)  //special demo 1,2,3,4 rotate mode
  {
   makeBrightPinsLow();   
   return;
  }

//5555555555555555555555555555555
solveNum5();   //5
delay(solveSpeed);

//9999999999999999999999999999999
solveNum9();   //9
delay(solveSpeed);

//13131313131313131313131313131313
solveNum13();  //13
delay(solveSpeed);

//6666666666666666666666666666666
solveNum6();   //6  
delay(solveSpeed);

//7777777777777777777777777777777
solveNum7();   //7
delay(solveSpeed);

//8888888888888888888888888888888
solveNum8();   //8
delay(solveSpeed);

//FOR THE 'SOLVED' PUZZLE#1 DEMO, JUST DO A BRIGHTENING SEQUENCE 14,10,11,12,15 
if ((digitalRead(Matrix1Pin)) == LOW)
  {
  specialFinalMovesDisplay();
  loop();
  }

//141414141414141414141414141414
solveNum14();  //14
delay(solveSpeed);

//101010101010101010101010101010
solveNum10();  //10
delay(solveSpeed);

//11 12 15 11 12 15 11 12 15 11 12 15 
solveFinal();  //final moves
//lcdDisplayElapsedDobotTime();

determinePosOf(15);
if (posOfNum15 == 15)
 {
 Move(17,17);    //flashOuterFrame and reset Display receive
 beep();
 beep();
 beep();
 }
else
 {
 flashNotSolvableMessage();   
 }

puzzleSolvedFlag = 1;

//Move(17,17);    //flashOuterFrame and reset Display receive
// beep();
// beep();
// beep();

}
// END OF SOLVE   END OF SOLVE   END OF SOLVE

//*************************************************************************
void flashNotSolvableMessage(){
  Serial1.write(254);
  Serial1.write(1);
for (int x=0;x<=6;x++)
  {   
  beep();
  Serial1.print("NOT Solvable!!!");
  delay(400);
  Serial1.write(254);
  Serial1.write(1);
  delay(100); 
  }
Serial1.print("NOT Solvable!!!");  
//while((digitalRead(sendArrayButton)) == HIGH);
//delay(2000);  //need delay so button doesn't ALSO trigger a restart
}

//*************************************************************************
void sendBrightness(int num){
//num = 1 to 15 (4 pins, 2,3,4,5)

for (int idx=0;idx<=3;idx++)  
  {
  int a = (1 << idx);  //bitwise shift, 1,2,4,8
  int b = (num & a);   //bitwise AND (num AND 1,2,4,8)
  if (b == a)          //if bit = true, write to pin
    {
    digitalWrite(brighteningNumberPinArray[idx], HIGH);
    }
    else
    {
    digitalWrite(brighteningNumberPinArray[idx], LOW);
    }
  }
}

//*******************************************************************************************
void waitForDisplayReadyToReceiveData(){
do
{
  x = digitalRead(DisplayReadyToReceiveData);
}
while(x == LOW);

}

//*******************************************************************************************
void transmitStartingMatrix(){
  
//transmitting 54 bytes in dec format, ("YY, or ZZ,") plus (1 = "01," = 48,49,44)  (15 = "15," = 49,53,44,)  (44 = comma)

if (interimImageFlag == HIGH)
   {
   Serial2.print("YY,");   //interim imageProcessing number locationss
   }
else
   {
   Serial2.print("ZZ,");   //final number locations
   }

for (int idx=0;idx<=16;idx++)            //sending 17 bytes
{
  if (startingMatrix[idx] <= 9)         //adds a leading "0" if Matrix[] value is 1,2,3,4,5,6,7,8,9
  {
  Serial2.print("0");
  }

  Serial2.print(startingMatrix[idx]);   //transmits 1 or 2 bytes, depending whether 1,2,3... or 11,12,13... I add a leading zero if it is 1 byte(i.e.<=9)
  Serial2.print(",");                   //I put the comma after every two bytes

  Serial.print(startingMatrix[idx]);
  Serial.print(",");
}
delay(1000);
}

//*******************************************************************************************
void sendMoveToDisplay(){    //but only when requested on input pin 43
 
//This routine is the end of the line for a single Move calculation. The Move is sent to the Display and is available to the Dobot
//using global From/To and local from/to variables

//########## THIS IS PLACE WHERE THE MOVE IS SENT TO THE DOBOT UNLESS THE MOVE IS JUST A BRIGHTENING ################  
//this calls a do loop that waits for CTS from the Dobot, latches data to pins, then lets program calculate next Move
//by releasing "sendMoveToDisplay()"  (note: sendMoveToDisplay incorporates "doPositionMove")
 
  if (sendDigitFlag == 0)   //if a brightening Move is being sent, the Dobot should not get it
    {
    checkIfDobotIsReadyForNextMove();
    }

//Sending Moves one at a time to Display, not Dobot here
//*******************************************************
//Special Header treatment
//sendDigitFlag & Digit or the Normal Move format MV,03,11, on TX2
//sendDigitFlag = 0;

if (sendDigitFlag == 1)
  { 
  if (Digit <= 9)
    {
    Serial2.print("0");
    }
  Serial2.print(Digit);
  Serial2.print(","); 
  sendDigitFlag = 0;  
  }
else
  {
  //Normal "Header" transmit
  Serial2.print("MV,");
  Serial.print("MV,");
  }
  
//************************************
//transmit "From" on Tx2
if (From <= 9)
 {
 Serial2.print("0");
 Serial.print("0");
 }
Serial2.print(From);
Serial.print(From);
Serial2.print(",");
Serial.print(",");

//************************************ 
//transmit "To" on Tx2 
if (To <= 9)
 {
 Serial2.print("0");
 Serial.print("0");
 }
Serial2.print(To);
Serial.print(To);
Serial2.print(",");
Serial.println(",");

blink();
}

//*********************************************************************************************
//    1111111111111111111111111111111111111111111111111111111111111111111111111111111111111
//*********************************************************************************************
void solveNum1(){

sendBrightness(1);   //sending number to brighten, using 4 discrete pins 
//but we must also tell the Display were the number is located,
//so we'll first send a Move with a format of (01,14,05,)
//which is the (number, X location, Y location). The usual Move starts with an "MV".
//Use a FLAG to tell the message to not send an MV. 
Digit = 1;
determinePosOf(1);
x = posOfNum1;
From = coordinatesInLEDMatrix[x][0];
To = coordinatesInLEDMatrix[x][1];
sendDigitFlag = 1;
sendMoveToDisplay();
//*********************************************** 
do
{
determinePosOf(1);  //also determines pos of num16      
//test for solve
if (posOfNum1 == 1)                                     
 {
 Serial.print("Passes = ");
 Serial.println(pass);
 display3DMatrix(); 
 return;
 }
//big scan of Num1array[256]
   for (Row=0;Row<=15;Row++)        
    {
      for (Pos=0;Pos<=15;Pos++)
       {
        if ((Row == (posOfNum1-1)) && (Pos == (posOfNum16-1)))   //the -1 compensates for the
                                                                 //BIG Num1array starting at 0
         {
          posToMoveToPosOf16 = Num1Array[(Row*16) + Pos];         
          doPositionMove();
          pass++;
         }
       }
     }
delay(10);
}
while(posOfNum1 != 1);    //going thru all 16 arrays having 16 positions each   
}

//*********************************************************************************************
//    2222222222222222222222222222222222222222222222222222222222222222222222222222222222222
//*********************************************************************************************

void solveNum2(){
//*************************************************************  
sendBrightness(2); 
Digit = 2;
determinePosOf(2);
x = posOfNum2;
From = coordinatesInLEDMatrix[x][0];
To = coordinatesInLEDMatrix[x][1];
sendDigitFlag = 1;
sendMoveToDisplay();   
//*************************************************************  
do
{
determinePosOf(2);  //also determines pos of num16      

//test for solve
if (posOfNum2 == 2)                                     
 {
 Serial.print("Passes = ");
 Serial.println(pass);
 display3DMatrix(); 
 return;
 }
   for (Row=0;Row<=15;Row++)        
    {
      for (Pos=0;Pos<=15;Pos++)
       {
       if ((Row == (posOfNum2-1)) && (Pos == (posOfNum16-1)))      
         {
          posToMoveToPosOf16 = Num2Array[(Row*16) + Pos];         
          doPositionMove();
          pass++;
         }
       }
     }
delay(10);
}
while(posOfNum2 != 2);    //going thru all 16 arrays having 16 positions each   
}

//*********************************************************************************************
//   33333333333333333333333333333333333333333333333333333333333333333333333333333333333333
//*********************************************************************************************

void solveNum3(){ 
//*************************************************************  
sendBrightness(3); 
Digit = 3;
determinePosOf(3);
x = posOfNum3;
From = coordinatesInLEDMatrix[x][0];
To = coordinatesInLEDMatrix[x][1];
sendDigitFlag = 1;
sendMoveToDisplay();   
//*************************************************************
do
{
determinePosOf(3);  //also determines pos of num16      
//test for solve
if (posOfNum3 == 3)                                     
 {
 Serial.print("Passes = ");
 Serial.println(pass);
 display3DMatrix(); 
 return;
 }
   for (Row=0;Row<=15;Row++)        
    {
      for (Pos=0;Pos<=15;Pos++)
       {
       if ((Row == (posOfNum3-1)) && (Pos == (posOfNum16-1)))      
         {
          posToMoveToPosOf16 = Num3Array[(Row*16) + Pos];         
          doPositionMove();
          pass++;
         }
       }
     }
delay(10);
}
while(posOfNum3 != 3);    //going thru all 16 arrays having 16 positions each   
}

//*********************************************************************************************
//  4444444444444444444444444444444444444444444444444444444444444444444444444444444444444444
//*********************************************************************************************

void solveNum4(){                                       

//*************************************************************  
sendBrightness(4); 
Digit = 4;
determinePosOf(4);
x = posOfNum4;
From = coordinatesInLEDMatrix[x][0];
To = coordinatesInLEDMatrix[x][1];
sendDigitFlag = 1;
sendMoveToDisplay();   
//*************************************************************
//initial of pos of num4, if either pos4 or pos8
determinePosOf(4);     //includes pos of num16 too

if (posOfNum4 == 4)
  {

  Serial.println("num4 solved at initialization");
  Serial.println("Stop");
  display3DMatrix();
  return;  //solveNum5();    
  Serial.println("goto solveNum5()");
  do
  {}
  while(1);
  }

if (posOfNum4 == 8) 
  {
  Serial.print("num16 = pos");
  Serial.println(posOfNum16);  
  specialMovesWhenNum4InPos8();
  Serial.println("num4 solved for Pos8 by SpecialMoves");
  Serial.println("Stop");
  display3DMatrix();
  Serial.println("goto solveNum5()");
  return;   //goto solveNum5();   
  }
//end of initial test for pos4 and pos8

//******************************************************************
//do the normal array scan, trying to get num4 in pos8, then use special moves

do
{
determinePosOf(4);  //also determines pos of num16      

//test for solve
if (posOfNum4 == 8)                                     
 {
 specialMovesWhenNum4InPos8();
 Serial.print("Passes = ");
 Serial.println(pass);
 display3DMatrix(); 
 return;
 }
 
//normal array scan follows
//*****************************************************
   for (Row=0;Row<=15;Row++)        
    {
      for (Pos=0;Pos<=15;Pos++)
       {
       if ((Row == (posOfNum4-1)) && (Pos == (posOfNum16-1)))      
         {
          posToMoveToPosOf16 = Num4Array[(Row*16) + Pos];         
          doPositionMove();
          pass++;
         }
       }
     }
delay(10);
}
while(posOfNum4 != 8);    //going thru all 16 arrays having 16 positions each   
//end of array scan, on to Specialmoves, discretes

}  //end of solveNum4

//*********************************************************************************************
//    5555555555555555555555555555555555555555555555555555555555555555555555555555555555555
//*********************************************************************************************

void solveNum5(){  
  
//*************************************************************  
sendBrightness(5); 
Digit = 5;
determinePosOf(5);
x = posOfNum5;
From = coordinatesInLEDMatrix[x][0];
To = coordinatesInLEDMatrix[x][1];
sendDigitFlag = 1;
sendMoveToDisplay();   
//*************************************************************
do
{
determinePosOf(5);  //also determines pos of num16      
//test for solve
if (posOfNum5 == 5)                                     
 {
 display3DMatrix(); 
 return;
 }
   for (Row=0;Row<=15;Row++)        
    {
      for (Pos=0;Pos<=15;Pos++)
       {
       if ((Row == (posOfNum5 - 1)) && (Pos == (posOfNum16 - 1)))     
         {
          posToMoveToPosOf16 = Num5Array[(Row*16) + Pos];         
          doPositionMove();
          pass++;
         }
       }
     }
delay(10);
}
while(posOfNum5 != 5);    //going thru all 16 arrays having 16 positions each   
}

//*********************************************************************************************
//   99999999999999999999999999999999999999999999999999999999999999999999999999999999999999
//*********************************************************************************************

void solveNum9(){ 
  
//*************************************************************  
sendBrightness(9); 
Digit = 9;
determinePosOf(9);
x = posOfNum9;
From = coordinatesInLEDMatrix[x][0];
To = coordinatesInLEDMatrix[x][1];
sendDigitFlag = 1;
sendMoveToDisplay();   
//*************************************************************
do
{
determinePosOf(9);  //also determines pos of num16      
//test for solve
if (posOfNum9 == 9)                                     
 {
 Serial.print("Passes = ");
 Serial.println(pass);
 display3DMatrix(); 
 return;
 }
   for (Row=0;Row<=15;Row++)        
    {
      for (Pos=0;Pos<=15;Pos++)
       {
       if ((Row == (posOfNum9 - 1)) && (Pos == (posOfNum16 - 1)))      
         {
          posToMoveToPosOf16 = Num9Array[(Row*16) + Pos];         
          doPositionMove();
          pass++;
         }
       }
     }
delay(10);
}
while(posOfNum9 != 9);    //going thru all 16 arrays having 16 positions each   
}

//*********************************************************************************************
//                           13 13 13 13 13 13 13 13 13 13 13 
//*********************************************************************************************

void solveNum13(){                                       

//*************************************************************  
sendBrightness(13); 
Digit = 13;
determinePosOf(13);
x = posOfNum13;
From = coordinatesInLEDMatrix[x][0];
To = coordinatesInLEDMatrix[x][1];
sendDigitFlag = 1;
sendMoveToDisplay();   
//*************************************************************  

//initial of pos of num13, if either pos13 or pos14
determinePosOf(13);     //includes pos of num16 too

if (posOfNum13 == 13)
  {
  Serial.println("num13 solved at initialization");
  Serial.println("Stop");
  display3DMatrix();
  return;                                   
  }
if (posOfNum13 == 14) 
  {
  Serial.print("num16 = pos");
  Serial.println(posOfNum16);  
  specialMovesWhenNum13InPos14AtInitial();
  Serial.println("num13 solved for Pos14 by SpecialMoves");
  Serial.println("Stop");
  display3DMatrix();
  return;                                      
  }
//end of initial test for pos13 and pos14

//******************************************************************
//do the normal array scan, trying to get num13 in pos14, then use special moves

do
{
determinePosOf(13);  //also determines pos of num16      

//test for solve
if (posOfNum13 == 14)                                     
 {
 specialMovesWhenNum13InPos14AtInitial();
 Serial.print("Passes = ");
 Serial.println(pass);
 display3DMatrix(); 
 return;
 }
 
//normal array scan follows
//*****************************************************
   for (Row=0;Row<=15;Row++)        
    {
      for (Pos=0;Pos<=15;Pos++)
       {
       if ((Row == (posOfNum13 - 1)) && (Pos == (posOfNum16 - 1)))      
         {
          posToMoveToPosOf16 = Num13Array[(Row*16) + Pos];         
          doPositionMove();
          pass++;
         }
       }
     }
delay(10);
}
while(posOfNum13 != 14);    //going thru all 16 arrays having 16 positions each   
//end of array scan, on to Specialmoves, discretes

}  //end of solveNum13

//*********************************************************************************************
//    6666666666666666666666666666666666666666666666666666666666666666666666666666666666666
//*********************************************************************************************

void solveNum6(){                                       

//*************************************************************  
sendBrightness(6); 
Digit = 6;
determinePosOf(6);
x = posOfNum6;
From = coordinatesInLEDMatrix[x][0];
To = coordinatesInLEDMatrix[x][1];
sendDigitFlag = 1;
sendMoveToDisplay();   
//*************************************************************
do
{
determinePosOf(6);  //also determines pos of num16      
//test for solve
if (posOfNum6 == 6)                                     
 {
 Serial.print("Passes = ");
 Serial.println(pass);
 display3DMatrix(); 
 return;
 }
   for (Row=0;Row<=15;Row++)        
    {
      for (Pos=0;Pos<=15;Pos++)
       {
       if ((Row == (posOfNum6 - 1)) && (Pos == (posOfNum16 - 1)))     
         {
          posToMoveToPosOf16 = Num6Array[(Row*16) + Pos];         
          doPositionMove();
          pass++;
         }
       }
     }
delay(10);
}
while(posOfNum6 != 6);    //going thru all 16 arrays having 16 positions each   
}

//*********************************************************************************************
//    7777777777777777777777777777777777777777777777777777777777777777777777777777777777777
//*********************************************************************************************

void solveNum7(){                                       

//*************************************************************  
sendBrightness(7); 
Digit = 7;
determinePosOf(7);
x = posOfNum7;
From = coordinatesInLEDMatrix[x][0];
To = coordinatesInLEDMatrix[x][1];
sendDigitFlag = 1;
sendMoveToDisplay();   
//*************************************************************
  
do
{
determinePosOf(7);  //also determines pos of num16      
//test for solve
if (posOfNum7 == 7)                                     
 {
 Serial.print("Passes = ");
 Serial.println(pass);
 display3DMatrix(); 
 return;
 }
   for (Row=0;Row<=15;Row++)        
    {
      for (Pos=0;Pos<=15;Pos++)
       {
       if ((Row == (posOfNum7 - 1)) && (Pos == (posOfNum16 - 1)))    
         {
          posToMoveToPosOf16 = Num7Array[(Row*16) + Pos];         
          doPositionMove();
          pass++;
         }
       }
     }
delay(10);
}
while(posOfNum7 != 7);    //going thru all 16 arrays having 16 positions each   
}

//*********************************************************************************************
//  88888888888888888888888888888888888888888888888888888888888888888888888888888888888888
//*********************************************************************************************

void solveNum8(){                                       

//*************************************************************  
sendBrightness(8); 
Digit = 8;
determinePosOf(8);
x = posOfNum8;
From = coordinatesInLEDMatrix[x][0];
To = coordinatesInLEDMatrix[x][1];
sendDigitFlag = 1;
sendMoveToDisplay();   
//************************************************************* 
 
//initial of pos of num8, if pos8                      
determinePosOf(8);     //includes pos of num16 too

if (posOfNum8 == 8)
  {
  Serial.println("num8 solved at initialization");
  Serial.println("Stop");
  display3DMatrix();
  return;                                   
  }

//******************************************************************
//do the normal array scan, geting num8 in pos12, then use special discrete moves

do
{
determinePosOf(8);  //also determines pos of num16      

//test for solve
if (posOfNum8 == 12)                                     
 {
 specialMovesWhenNum8InPos12();
 Serial.print("Passes = ");
 Serial.println(pass);
 display3DMatrix(); 
 return;
 }
 
//normal array scan follows
//*****************************************************
   for (Row=0;Row<=15;Row++)        
    {
      for (Pos=0;Pos<=15;Pos++)
       {
       if ((Row == (posOfNum8-1)) && (Pos == (posOfNum16-1)))      
         {
          posToMoveToPosOf16 = Num8Array[(Row*16) + Pos];         
          doPositionMove();
          pass++;
         }
       }
     }
delay(10);
}
//while(posOfNum8 != 12);
while(1);

//going thru all 16 arrays having 16 positions each   
//end of array scan, on to Specialmoves, discretes

}  //end of solveNum8


//*********************************************************************************************
//  14141414141414141414141414141414141414141414141414141414141414141414141414141414141414
//*********************************************************************************************
void solveNum14(){      //must put it in Pos10, under Num6

brightenNum14();

//*************************************************************
do
{
determinePosOf(14);  //also determines pos of num16      

if (posOfNum14 == 10)                                     
 {
 Serial.print("Passes = ");
 Serial.println(pass);
 display3DMatrix(); 
 return;
 }
   for (Row=0;Row<=15;Row++)        
    {
      for (Pos=0;Pos<=15;Pos++)
       {
       if ((Row == (posOfNum14 - 1)) && (Pos == (posOfNum16 - 1)))     
         {
          posToMoveToPosOf16 = Num14toPos10Array[(Row*16) + Pos];        
          doPositionMove();
          pass++;
         }
       }
     }
delay(10);
}
while(posOfNum14 != 10);    //going thru all 16 arrays having 16 positions each   
}


//*********************************************************************************************
//  10101010101010101010101010101010101010101010101010101010101010101010101010101010101010
//*********************************************************************************************
void solveNum10(){     //put it to the right of the num14, in pos11, under the num7 

brightenNum10();

//*************************************************************

determinePosOf(10);   //and posNum16 too
//determinePosOf(14);

//if 14 and 10 are already in position, there is no reason to do
//the around the corner at the bottom of this routine.  
//i.e. if ((posOfNum10 == 10) && (posOfNum14 == 14))
//       {  
//       return;
//       }

if (posOfNum10 == 11){
  //We have to move num16 to pos14 so we can rotate 14 & 10 around the corner
  switch (posOfNum16){
    case 12:{Move(16,12); Move(15,16); Move(14,15); break;}
    case 14:{break;}
    case 15:{Move(14,15); break;}
    case 16:{Move(15,16); Move(14,15); break;}      
  }
}

//*************************************
if (posOfNum10 == 12){
  switch (posOfNum16){
     case 11:{Move(12,11); Move(16,12); Move(15,16); Move(14,15); break;}
     case 14:{Move(15,14); discrete10_12(); break;}
     case 15:{discrete10_12(); break;}
     case 16:{Move(15,16); discrete10_12(); break;}
  }
}

//*************************************
if (posOfNum10 == 14){
  switch (posOfNum16){
     case 11:{Move(15,11); discrete10_14(); break;}
     case 12:{Move(11,12); Move(15,11); discrete10_14(); break;}
     case 15:{discrete10_14(); break;}
     case 16:{Move(15,16); discrete10_14(); break;}
  }
}

//*************************************
if (posOfNum10 == 15){
  switch (posOfNum16){
     case 11:{Move(15,11); Move(14,15); break;}
     case 12:{Move(11,12); Move(15,11); Move(14,15); break;}
     case 14:{Move(10,14); Move(11,10); Move(15,11); Move(16,15);\
              Move(12,16); Move(11,12); Move(10,11); Move(14,10);\
              Move(15,14); Move(11,15); Move(12,11); Move(16,12);\
              Move(15,16); Move(14,15); break;}
     case 16:{Move(12,16); Move(11,12); Move(15,11); Move(14,15); break;}
  }
}

//*************************************
if (posOfNum10 == 16){
  switch (posOfNum16){
     case 11:{Move(12,11); discrete10_16(); break;}
     case 12:{discrete10_16(); break;}
     case 14:{Move(15,14); Move(11,15); Move(12,11); discrete10_16(); break;}
     case 15:{Move(16,15); Move(12,16); Move(11,12); Move(15,11); Move(14,15); break;}
  }
}

//finally, move 14 and 10 around the corner
   Move(10,14); Move(11,10);
   display3DMatrix();
   return;

}   //END OF SOLVE NUM10, with NUM14 already in POS14}

//*********************************************************
//   Additional discrete Move routines for solving num10
//*********************************************************
void discrete10_12(){
  Move(11,15); Move(12,11); Move(16,12); Move(15,16); Move(14,15);
}

//*************************************
void discrete10_14(){
  Move(14,15); Move(10,14); Move(11,10); Move(15,11); Move(16,15); Move(12,16);\
  Move(11,12); Move(10,11); Move(14,10); Move(15,14); Move(11,15);\
  Move(12,11); Move(16,12); Move(15,16); Move(14,15);        
}

//*************************************
void discrete10_16(){
  Move(16,12); Move(15,16); Move(11,15); Move(12,11); Move(16,12);\
  Move(15,16); Move(14,15); 
}

//*************************************
void move10(byte from, byte to){
    Matrix[to] = Matrix[from];
    Matrix[from] = 16;
    From = from;
    To = to;
    sendMoveToDisplay();
    moves++;
}


//*********************************************************************************************
//                              SOLVING 11,12,15 FINAL MOVES       
//                     (STARTING WITH 10 AND 14 IN THEIR FINAL LOCATIONS)     
//*********************************************************************************************
void solveFinal(){     //solving only 11,12 & 15, located in Positions 11, 12, 15, or 16

//*************************************************************
brightenNum11();
brightenNum12();
brightenNum15();
//*************************************************************
  
determinePosOf(11);   //and posNum16 too

if (posOfNum11 == 11){
  switch (posOfNum16){
    case 12:{Move(16,12); break; }     
    case 15:{Move(16,15); break; }
    case 16:{break;}  
  }
display3DMatrix();
makeBrightPinsLow();  
return;   
}
//*****************************************
if (posOfNum11 == 12){
  switch (posOfNum16){
    case 11:{discrete11_12(); break;}
    case 15:{Move(11,15); discrete11_12(); break;}
    case 16:{Move(15,16); Move(11,15); discrete11_12(); break;}
  }
display3DMatrix();
makeBrightPinsLow();
return;   
}
//*****************************************
if (posOfNum11 == 15){
  switch (posOfNum16){
    case 11:{discrete11_15(); break;}
    case 12:{Move(11,12); discrete11_15(); break;}
    case 16:{Move(12,16); Move(11,12); discrete11_15(); break;}
   }
display3DMatrix();
makeBrightPinsLow();
return;   
}

//*****************************************
if (posOfNum11 == 16){
  switch (posOfNum16){
    case 11:{Move(15,11); discrete11_16(); break;}
    case 12:{Move(11,12); Move(15,11); discrete11_16(); break;}
    case 15:{discrete11_16(); break;}
   }
display3DMatrix(); 
makeBrightPinsLow();
return;
}   

}   //END OF FINAL MOVES, END OF COMPLETE PUZZLE SOLUTION
//********************************************************************************
//********************************************************************************
//********************************************************************************


//****************************************************
//      Discrete Move routines for solving num11
//****************************************************
void discrete11_12(){
  Move(12,11); Move(16,12);
}
//*************************************
void discrete11_15(){
  Move(15,11); Move(16,15);         
}
//*************************************
void discrete11_16(){
  Move(16,15); Move(12,16); Move(11,12); Move(15,11); Move(16,15);
}

//******************************************************************
//               FINAL MOVES, OTHER SUPPORT ROUTINES
//******************************************************************
void makeBrightPinsLow(){
digitalWrite(brightOnePin, LOW);
digitalWrite(brightTwoPin, LOW);
digitalWrite(brightFourPin, LOW);
digitalWrite(brightEightPin, LOW);
} 
 
//*****************************************************
void specialFinalMovesDisplay(){
//brightening 14, delay, 10, delay, 11,12,15
//delay(500);
brightenNum14();
delay(1000);
brightenNum10();
delay(1000);
brightenNum11();
delay(1000);
brightenNum12();
delay(1000);
brightenNum15();
delay(500);
Move(17,17);    //flashOuterFrame and reset Display receive
do
{}
while(1); 
}

//**********************************************************************************
//                 EVEN MORE SUPPORT FUNCTIONS
//**********************************************************************************

void move11(byte from, byte to){
    Matrix[to] = Matrix[from];
    Matrix[from] = 16;
    From = from;
    To = to;
    sendMoveToDisplay();
    moves++;
}

//************************************
void brightenNum10(){    
sendBrightness(10);    //this only sets the output pins 
Digit = 10;
determinePosOf(10);
x = posOfNum10;
From = coordinatesInLEDMatrix[x][0];
To = coordinatesInLEDMatrix[x][1];
sendDigitFlag = 1;
sendMoveToDisplay();  
}

//************************************
void brightenNum11(){    
sendBrightness(11); 
Digit = 11;
determinePosOf(11);
x = posOfNum11;
From = coordinatesInLEDMatrix[x][0];
To = coordinatesInLEDMatrix[x][1];
sendDigitFlag = 1;
sendMoveToDisplay();  
}

//*************************************
void brightenNum12(){
sendBrightness(12); 
Digit = 12;
determinePosOf(12);
x = posOfNum12;
From = coordinatesInLEDMatrix[x][0];
To = coordinatesInLEDMatrix[x][1];
sendDigitFlag = 1;
sendMoveToDisplay();  
}

//************************************
void brightenNum14(){    
sendBrightness(14); 
Digit = 14;
determinePosOf(14);
x = posOfNum14;
From = coordinatesInLEDMatrix[x][0];
To = coordinatesInLEDMatrix[x][1];
sendDigitFlag = 1;
sendMoveToDisplay();  
}

//*************************************
void brightenNum15(){
sendBrightness(15); 
Digit = 15;
determinePosOf(15);
x = posOfNum15;
From = coordinatesInLEDMatrix[x][0];
To = coordinatesInLEDMatrix[x][1];
sendDigitFlag = 1;
sendMoveToDisplay();  
}

//**********************************************************************************************************************
//**********************************************************************************************************************
//                                     GENERAL AND SPECIAL FUNCTIONS AND ROUTINES
//**********************************************************************************************************************
//**********************************************************************************************************************

//***********************************************************************************
//                     SPECIAL ROUTINES FOR NUM4 ONLY
//***********************************************************************************
void specialMovesWhenNum4InPos8(){
  
  determinePosOfNum16_4();
  
  switch (posOfNum16){
    case 4: {Move(8,4); break; }
    case 5: {Move(6,5); discrete4(); break; }
    case 6: {discrete4(); break; }  
    case 7: {Move(6,7); discrete4(); break; }
  //case 8: num4 is in pos8
    case 9: {Move(5,9); Move(6,5); discrete4(); break; }
    case 10:{Move(6,10); discrete4(); break; }  
    case 11:{Move(10,11); Move(6,10); discrete4(); break; }
    case 12:{Move(11,12); Move(10,11); Move(6,10); discrete4(); break; }
    case 13:{Move(14,13); Move(10,14); Move(6,10); discrete4(); break; }
    case 14:{Move(10,14); Move(6,10); discrete4(); break; }
    case 15:{Move(14,15); Move(10,14); Move(6,10); discrete4(); break; } 
    case 16:{Move(15,16); Move(14,15); Move(10,14); Move(6,10); discrete4(); break; }
    default:{break;}
    }
}
//************************************************
void determinePosOfNum16_4() {
  for (int idx=1;idx<=16;idx++){
    if (Matrix[idx] == 16)(posOfNum16 = idx);
    }
}
//************************************************
void discrete4(){
 //with 16 in pos7(Move(6,7)has been done), do three rotates total, CCW,CCW,CW 
 Move(2,6); Move(3,2); Move(4,3); Move(8,4); Move(7,8); Move(3,7); Move(2,3); Move(6,2);  //done correctly
 }

//************************************************
void Move(byte from, byte to){
  
    Matrix[to] = Matrix[from];
    Matrix[from] = 16;

    if ((digitalRead(Matrix3Pin)) == LOW)  //slow down solve speed for demo of 1,2,3,4 moves
       {delay(Matrix3PinDelay);}

    if ((digitalRead(Matrix5Pin)) == LOW)  //slow down solve speed for demo of 14,10 moves
       {delay(Matrix5PinDelay);}
 
    From = from;
    To = to;
    sendMoveToDisplay();
    moves++;
}

//**************************************************************************************
//                   SPECIAL ROUTINES FOR NUM8 ONLY
//**************************************************************************************

void specialMovesWhenNum8InPos12(){     //when num8Array puts num8 in pos12 
  determinePosOfNum16_8();
   
  switch(posOfNum16){
    case 8: {Move8(12,8); break;}
    case 10:{discrete8(); break;}   //using discrete8 as a abbreviation of num8with16atPos10DiscreteMoves()
    case 11:{Move8(10,11); discrete8(); break;}
    case 14:{Move8(10,14); discrete8(); break;}
    case 15:{Move8(14,15); Move8(10,14); discrete8(); break;} 
    case 16:{Move8(15,16); Move8(14,15); Move8(10,14); discrete8() ;break;}
    return;
    }
    return;
}

//************************************************
void determinePosOfNum16_8() {
  for (int idx=1;idx<=16;idx++){
    if (Matrix[idx] == 16)(posOfNum16 = idx);
    }
}

//************************************************
void discrete8(){
 num8with16atPos10DiscreteMoves();
}

//************************************************
void Move8(byte from, byte to){
    Matrix[to] = Matrix[from];
    Matrix[from] = 16;
    From = from;
    To = to;
    sendMoveToDisplay();
    moves++;
}

//******************************************************************
//8 discrete moves
void num8with16atPos10DiscreteMoves(){
for (idx=0;idx<=7;idx++){
         
  From = Num8toPos8WhenNum8inPos12andNum16inPos10Array[idx][0]; 
  To =   Num8toPos8WhenNum8inPos12andNum16inPos10Array[idx][1];

  Matrix[To] = Matrix[From];
  Matrix[From] = 16;
  sendMoveToDisplay();
  }
}

//**************************************************************************************
//                      SPECIAL ROUTINES FOR NUM13 ONLY
//**************************************************************************************

void specialMovesWhenNum13InPos14AtInitial(){    //AT INITIALIZATION ONLY!!!!!!!!!!!!!!!!!!!!
  determinePosOfNum16_13();
  //NOTE: THIS SEQUENCE MOVES NUM16 TO POS 10, FOR ALL INITIAL POSITIONS OF 16,
  //      THEN APPLIES THE STANDARD NINE(9) DISCRETE AROUND-THE-CORNER MOVES
  switch (posOfNum16){
    case 6: {Move13(10,6); discrete13(); break; }
    case 7: {Move13(6,7); Move13(10,6); discrete13(); break; }
    case 8: {Move13(7,8); Move13(6,7); Move13(10,6); discrete13(); break; }
    case 10:{discrete13(); break; }   //using discrete13 as a convenient abbreviation of num13with16atPos10DiscreteMoves()
    case 11:{Move13(10,11); discrete13(); break; }
    case 12:{Move13(11,12); Move13(10,11); discrete13(); break; }
    case 13:{Move13(14,13); break; }  
    case 15:{Move13(11,15); Move13(10,11); discrete13(); break; } 
    case 16:{Move13(15,16); Move13(11,15); Move13(10,11); discrete13(); break; }
    break;
    }
}

//************************************************
void determinePosOfNum16_13() {
  for (int idx=1;idx<=16;idx++){
    if (Matrix[idx] == 16)(posOfNum16 = idx);
    }
}

//************************************************
void discrete13(){                                
 num13with16atPos10DiscreteMoves();
}

//************************************************
void Move13(byte from, byte to){                    

    Matrix[to] = Matrix[from];
    Matrix[from] = 16;
    From = from;
    To = to;    
    sendMoveToDisplay();
    moves++;
    
}

//*****************************************************************
void num13with16atPos15DiscreteMoves(){                            
for (idx=0;idx<=1;idx++){                                         
       //Num13toPos13WhenNum13inPos14andNum16inPos15Array  Okay!
  From = Num13toPos13WhenNum13inPos14andNum16inPos15Array[idx][0];   
  To =   Num13toPos13WhenNum13inPos14andNum16inPos15Array[idx][1];     

  Matrix[To] = Matrix[From];
  Matrix[From] = 16;
  sendMoveToDisplay();
  }
}

//******************************************************************
     
void num13with16atPos10DiscreteMoves(){                               
for (idx=0;idx<=8;idx++){                                             
  //9 discrete around-the-corner moves       
  From = Num13toPos13WhenNum13inPos14andNum16inPos10Array[idx][0];       
  To =   Num13toPos13WhenNum13inPos14andNum16inPos10Array[idx][1];       

  Matrix[To] = Matrix[From];
  Matrix[From] = 16;
  sendMoveToDisplay();
  }
}


//******************************************************************
//******************************************************************
//           GENERAL ROUTINES FOR NUMBERS
//******************************************************************
//******************************************************************

//*******************************************************************
void determinePosOf(int Num){    //find pos of NumX and Num16 using primary Matrix[] array

for (int x=1;x<=16;x++){                    
 if (Matrix[x] == Num)(posOfNum1 = x);
 if (Matrix[x] == Num)(posOfNum2 = x); 
 if (Matrix[x] == Num)(posOfNum3 = x);
 if (Matrix[x] == Num)(posOfNum4 = x);
 if (Matrix[x] == Num)(posOfNum5 = x); 
 if (Matrix[x] == Num)(posOfNum6 = x);
 if (Matrix[x] == Num)(posOfNum7 = x);
 if (Matrix[x] == Num)(posOfNum8 = x); 
 if (Matrix[x] == Num)(posOfNum9 = x);
 if (Matrix[x] == Num)(posOfNum10 = x);
 if (Matrix[x] == Num)(posOfNum11 = x); 
 if (Matrix[x] == Num)(posOfNum12 = x);
 if (Matrix[x] == Num)(posOfNum13 = x);
 if (Matrix[x] == Num)(posOfNum14 = x); 
 if (Matrix[x] == Num)(posOfNum15 = x);
 if (Matrix[x] == 16)(posOfNum16 = x);
 }
 return(x);
}

//***************************************************************
void doPositionMove(){
  
Matrix[posOfNum16]= Matrix[posToMoveToPosOf16];   //To = From
Matrix[posToMoveToPosOf16]= 16;

From = posToMoveToPosOf16;
To = posOfNum16;
moves++;

sendMoveToDisplay();

blink();
}

//***************************************************************
void displayMatrix(){
for (int idx=1;idx<=16;idx++)
  {
  Serial.print(Matrix[idx]);
  Serial.print(",");
  }
Serial.println("");
}

//***************************************************************
void loadStartingMatrixInMatrix(){
for (int idx=0;idx<=16;idx++)
  {
    Matrix[idx] = startingMatrix[idx];
  }
}

//**************************************************************
void display3DMatrix(){
  //Not used since the early 1800's
}

//*******************************************************
void setPuzzleNumLEDs(int LED){

for (int idx=0;idx<=2;idx++)  
  {
  int a = (1 << idx);  //bitwise shift, 1,2,4
  int b = (LED & a);   //bitwise AND (LED AND 1,2,4)
  if (b == a)          //if bit = true, write to pin
    {
    digitalWrite(puzzleNumberLEDPinArray[idx], HIGH);
    }
    else
    {
    digitalWrite(puzzleNumberLEDPinArray[idx], LOW);
    }
  }
}

//*******************************************************
void beep(){
  digitalWrite(beeper, HIGH);
  delay(100);
  digitalWrite(beeper, LOW);
  delay(100);
  }
  
//*******************************************************
void blink()   //blink RED
{
digitalWrite(LED, HIGH);
delay(50);
digitalWrite(LED, LOW);
delay(50);
}

void blinkGrn()
{
digitalWrite(LEDGrn, HIGH);
delay(50);
digitalWrite(LEDGrn, LOW);
delay(50);
}

void blinkBlu()
{
digitalWrite(LEDBlu, HIGH);
delay(50);
digitalWrite(LEDBlu, LOW);
delay(50);
}

void blinkYel()
{
digitalWrite(LEDYel, HIGH);
delay(50);
digitalWrite(LEDYel, LOW);
delay(50);
}

void YelOn()
{digitalWrite(LEDYel, HIGH);}
void YelOff()
{digitalWrite(LEDYel, LOW);}

void BluOn()
{digitalWrite(LEDBlu, HIGH);}
void BluOff()
{digitalWrite(LEDBlu, LOW);}

void GrnOn()
{digitalWrite(LEDGrn, HIGH);}
void GrnOff()
{digitalWrite(LEDGrn, LOW);}

//**********************************************************************************
void solveImageDemo()   //same as hitting DO button
   {
   beep();
    
   blink();  //red
   fastSolve = 0;
   programMode = 1;        //all-up operation, with Dobot moving puzzle pieces and Display sync'd to moving pieces
   puzzleSolvedFlag = 0;   //this is true start of the whole program(s)
   //digitalWrite(blankEaselPin, LOW);
   rstDisplayArduino();
    beep();
    beep();
    digitalWrite(goCmdToRaspiPin, HIGH);  //trigger to Raspi
    delay(1000);
    digitalWrite(goCmdToRaspiPin, LOW);
   
   //elapsed time display
   iteration = 1;
   iteration2 = 0;
   startTime = millis();

   do    //do loop to display Matt's image solving sequence, then final startingMatrix
   {
    loadImageMatrixFromRaspi();   //BIG do loop waiting for serial data from Raspi matching routine
   
    sendArrayToDisplay();   
    iteration2++;
    delay(1000);  //important delay so 32x32 doesn't get overwritten
    beep();
   }
    while(iteration2 != 15);   //loop stops when Matt has completely determined configuration
 
   delay(3000);  //delay so observer can view elapsed time
  
   do     //wait for Big Yellow to start the Dobot moving
   {
   digitalWrite(bigYellowLEDPin, HIGH); 
   Serial1.write(254);
   Serial1.write(1);
   delay(15);
   Serial1.print("Confirm Match");
   Serial1.write(254);
   Serial1.write(192);
   Serial1.print("Then Hit Yellow");
   delay(500);
   digitalWrite(bigYellowLEDPin, LOW); 
   Serial1.write(254);
   Serial1.write(1);  //clear LCD
   delay(200);
   }
   while((digitalRead(bigYellowButtonPin)) == HIGH);  //used to be Dobot button 

   digitalWrite(bigYellowLEDPin, LOW);
   delay(2000);

   //#######################################    
   digitalWrite(RequestToSendSolveMoves, HIGH); 
   waitForDisplayReadyToReceiveData();
   digitalWrite(RequestToSendSolveMoves, LOW);
   //###################################### 
    
   doSolving();   //this solves until puzzleSolvedFlag = 1
 } 

//***********************************************************

void lcdDisplayElapsedDobotTime(){
  
  long currentTime = millis();
  long elapsedTime = (currentTime - dobotStartTime);

  int minTime = ((elapsedTime/1000)/60);
  int secTime = (elapsedTime/1000)%60;

  Serial1.write(254);
  Serial1.write(192);  //line 2

  Serial1.print("Time: ");
  Serial1.print(minTime);
  Serial1.print(":");
  if (secTime <= 9)(Serial1.print("0"));
  Serial1.print(secTime);
  Serial1.print(" min");
  
}

//end of program
