import RPi.GPIO as GPIO
import time
import config

def get_setting(key):
    return config.get_setting('gpio', key).strip('"')

def get_setting_int(key):
    return int(get_setting(key))

def get_setting_trig_type(key):
    trig_val = get_setting(key)
    map = {
        'FALLING': GPIO.FALLING,
        'RISING': GPIO.RISING
    }
    return int(map.get(trig_val))

def init(on_go, on_reset, on_mode):
       
    GPIO.setmode(GPIO.BCM)
    GPIO.setwarnings(False)
    
    GPIO.setup(LED_READY_PIN, GPIO.OUT)
    GPIO.setup(LED_PROCESSING_PIN, GPIO.OUT)
    GPIO.setup(LED_ERROR_PIN, GPIO.OUT)
    
    GPIO.setup(BEEPER_PIN, GPIO.OUT) 
    GPIO.setup(PHOTO_LIGHT_PIN, GPIO.OUT)
    
    GPIO.setup(GO_BTN_PIN, GPIO.IN)     
    GPIO.setup(GO_SIG_PIN, GPIO.IN)
    GPIO.setup(RESET_PIN, GPIO.IN)
    GPIO.setup(MODE_PIN, GPIO.IN)

    time.sleep(2)            #stabilize LOW outputs to LEDs
    
    GPIO.add_event_detect(GO_BTN_PIN, get_setting_trig_type('GO_BTN_TRIG_TYPE'), callback=on_go, bouncetime=500)
    GPIO.add_event_detect(GO_SIG_PIN, get_setting_trig_type('GO_SIG_TRIG_TYPE'), callback=on_go, bouncetime=500)
        # !! not tested ^^
    GPIO.add_event_detect(RESET_PIN, get_setting_trig_type('RESET_TRIG_TYPE'), callback=on_reset, bouncetime=500)
    GPIO.add_event_detect(MODE_PIN, get_setting_trig_type('MODE_TRIG_TYPE'), callback=on_mode, bouncetime=500)
    
    return GPIO

def led_change(pin, value):
    GPIO.output(pin, value)

def led_ready_on(mutually_exclusive = True):
    if (mutually_exclusive):
        reset_all_leds()   
    led_change(LED_READY_PIN, get_setting_int('LED_READY_ON'))
    
def led_ready_off():
    led_change(LED_READY_PIN, get_setting_int('LED_READY_OFF'))
    
def led_processing_on(mutually_exclusive = True):
    if (mutually_exclusive):
        reset_all_leds()
    led_change(LED_PROCESSING_PIN, get_setting_int('LED_PROCESSING_ON'))
    
def led_processing_off():
    led_change(LED_PROCESSING_PIN, get_setting_int('LED_PROCESSING_OFF'))

def led_processing_blink():
    led_processing_off()
    time.sleep(0.1)
    led_processing_on()

def led_error_on(mutually_exclusive = True):
    if (mutually_exclusive):
        reset_all_leds()
    led_change(LED_ERROR_PIN, get_setting_int('LED_ERROR_ON'))
    
def led_error_off():
    led_change(LED_ERROR_PIN, get_setting_int('LED_ERROR_OFF'))

def beeper_short_beep():
    beeper_on(1/20)
    beeper_off()

def beeper_on(duration):
    # Duration in seconds
    GPIO.output(BEEPER_PIN, get_setting_int('BEEPER_ON'))    
    time.sleep(duration)
    beeper_off()
   
def beeper_off():
    GPIO.output(BEEPER_PIN, get_setting_int('BEEPER_OFF'))  

def photo_light_on():
    GPIO.output(PHOTO_LIGHT_PIN, get_setting_int('PHOTO_LIGHT_ON'))
    
def photo_light_off():
    GPIO.output(PHOTO_LIGHT_PIN, get_setting_int('PHOTO_LIGHT_OFF'))

def is_mode_adj():
    # Adj mode when low
    return not GPIO.input(MODE_PIN)

def is_mode_run():
    # Run mode when high
    return not not GPIO.input(MODE_PIN)

def is_reset_pressed():
    return GPIO.input(RESET_PIN) == get_setting_int('RESET_PIN_PRESSED')

def reset_all_leds():
    led_ready_off()
    led_processing_off()
    led_error_off()

def reset_all():
    reset_all_leds()
    beeper_off()
    photo_light_off()


BEEPER_PIN = get_setting_int('BEEPER_PIN')
GO_BTN_PIN = get_setting_int('GO_BTN_PIN')
GO_SIG_PIN = get_setting_int('GO_SIG_PIN')
LED_ERROR_PIN = get_setting_int('LED_ERROR_PIN')
LED_PROCESSING_PIN = get_setting_int('LED_PROCESSING_PIN')
LED_READY_PIN = get_setting_int('LED_READY_PIN')
MODE_PIN = get_setting_int('MODE_PIN')
PHOTO_LIGHT_PIN = get_setting_int('PHOTO_LIGHT_PIN')
RESET_PIN = get_setting_int('RESET_PIN')
